# -----------------------------------------------------------------
# Copyright (c) 2022, AIT Austrian Institute of Technology GmbH.
# All rights reserved. See file FMIPP_LICENSE.txt for details.
# -----------------------------------------------------------------

#
# This file is used to create FMUs for Co-Simulation from Python code.
#

### Setup for Python 2.
try:
    import sys, os, shutil, time, getpass, uuid, pickle, subprocess, glob, urlparse, urllib, collections, inspect, warnings, platform
except:
    pass

### Setup for Python 3.
try:
    import sys, os, shutil, time, getpass, uuid, pickle, subprocess, glob, urllib.parse as urlparse, urllib.request as urllib, collections, inspect, warnings, platform
except:
    pass

def log( *arg ):
    print( ' '.join( map( str, arg ) ) )
    sys.stdout.flush()


from fmipp.export.FMIAdapterBase import FMIAdapterBase
from fmipp.export.fmi1 import *
from fmipp.export.fmi2 import *

def createFMU(
    fmu_backend,
    fmi_model_identifier,
    fmi_version = '2',
    verbose = False,
    litter = False,
    start_values = None,
    optional_files = None ):
    """
    Create and FMU for Co-Simulation for a class derived from FMIAdapter.

    :param fmu_backend: class implementing the abstract base class FMIAdapter (class derived from FMIAdapter)
    :param fmi_model_identifier: FMI model identifier (str)
    :param fmi_version: FMI version (str, 1 or 2, default: 2)
    :param verbose: turn on log messages (boolean, default: False)
    :param litter: do not clean-up intermediate files (boolean, default: False)
    :param start_values: start values may be specified for paramters and input variables (None or dict, default: None)
    :param optional_files: additional files (e.g., for weather data) may be specified as extra arguments; these files will be automatically copied to the resources directory of the FMU (None or list of str, default: None)
    """

    # Check FMI model identifier.
    if not isinstance( fmi_model_identifier, str ):
        raise TypeError( 'parameter \'fmi_model_identifier\' must be of type \'str\'' )

    # Check FMI model identifier.
    if not fmi_version in [ '1', '2' ]:
        raise TypeError( 'parameter \'fmi_version\' must be either 1 or 2' )

    # Check verbosity flag.
    if not isinstance( verbose, bool ):
        raise TypeError( 'parameter \'verbose\' must be of type \'bool\'' )

    # Check verbosity flag.
    if not isinstance( litter, bool ):
        raise TypeError( 'parameter \'litter\' must be of type \'bool\'' )

    # Check start values.
    if start_values is None:
        start_values = {}
    elif not isinstance( start_values, dict ):
        raise TypeError( 'parameter \'start_values\' must be \'None\' or of type \'dict\'' )

    # Check optional files.
    if optional_files is None:
        optional_files = []
    elif not isinstance( optional_files, list ) and not all( isinstance( elem, str ) for elem in optional_files ):
        raise TypeError( 'parameter \'optional_files\' must be \'None\' or of type \'list of str\'' )

    # Create container for all used Python modules, which will be passed to all called functions.
    # This makes it easier to run this script with different Python version (2.x and 3.x).
    Modules = collections.namedtuple( 'Modules', [ 'sys', 'os', 'shutil', 'time', 'getpass', 'uuid', 'urlparse', 'urllib', 'pickle', 'subprocess', 'glob', 'log', 'inspect', 'warnings', 'platform' ] )
    modules = Modules( sys, os, shutil, time, getpass, uuid, urlparse, urllib, pickle, subprocess, glob, log, inspect, warnings, platform )

    if ( True == verbose ):
        modules.log( '[DEBUG] Using FMI version {}'.format( fmi_version ) )
        modules.log( '[DEBUG] FMI model identifier: {}'.format( fmi_model_identifier ) )
        modules.log( '[DEBUG] Aditional files:' )
        for file_name in optional_files:
            modules.log( '\t{}'.format( file_name ) )

    # Inspect implementation of FMU back-end to retrieve FMI input and output variable names.
    ( fmi_input_vars, fmi_output_vars, fmi_parameters, fmu_backend_class, fmu_backend_file ) = inspectFMUBackend( fmu_backend, verbose, modules )

    try:
        fmu_name = generateFMU(
            fmi_version,
            fmi_model_identifier,
            fmu_backend_class,
            fmu_backend_file,
            fmi_input_vars,
            fmi_output_vars,
            fmi_parameters,
            start_values,
            optional_files,
            verbose,
            litter,
            modules )

        if ( True == verbose ): modules.log( "[DEBUG] FMU created successfully:", fmu_name )

    except Exception as e:
        modules.log( e )
        modules.sys.exit( e.args[0] )


def inspectFMUBackend( fmu_backend, verbose, modules ):
    """
    Check if backend inherits from FMIAdapter.
    Retrieve the name of the file in which the backend is defined.
    Retrieve the input variables, output variables and parameter that an FMU backend implementation defines.
    """
    # Check FMU backend implementation.
    if not issubclass( fmu_backend, FMIAdapterBase ):
        raise TypeError( 'class \'fmu_backend\' must be a subclass of \'FMIAdapter\'' )

    # Dicts containing information about the FMI input variables, output variables and parameters.
    fmi_input_vars = {}
    fmi_output_vars = {}
    fmi_parameters = {}

    # Instantiate the backend (debug mode).
    backend = fmu_backend()

    fmu_backend_class = backend.__class__.__name__

    # Get file in which the backend class is defined.
    fmu_backend_file = str()
    try:
        fmu_backend_file = modules.inspect.getsourcefile( backend.__class__ )

        if ( True is verbose ):
            modules.log( '[DEBUG] Class \'{0}\' defined in file: {1}'.format( fmu_backend_class, fmu_backend_file ) )
    except TypeError:
        raise RuntimeError( 'File containing definition of class \'{}\' not found'.format( fmu_backend_class )  )

    with modules.warnings.catch_warnings():
        modules.warnings.simplefilter( "ignore", category = RuntimeWarning )
        # Initialize the backend.
        backend.init( 0 )

        fmi_input_vars[ 'Real' ] = backend._realInputNames
        fmi_input_vars[ 'Integer' ] = backend._integerInputNames
        fmi_input_vars[ 'Boolean' ] = backend._booleanInputNames
        fmi_input_vars[ 'String' ] = backend._stringInputNames

        if ( True == verbose ):
            modules.log( '[DEBUG] FMI input variables:' )
            for type, vars in fmi_input_vars.items():
                for var in vars:
                    modules.log( '\t{0} ({1})'.format( var, type ) )

        fmi_output_vars[ 'Real' ] = backend._realOutputNames
        fmi_output_vars[ 'Integer' ] = backend._integerOutputNames
        fmi_output_vars[ 'Boolean' ] = backend._booleanOutputNames
        fmi_output_vars[ 'String' ] = backend._stringOutputNames

        if ( True == verbose ):
            modules.log( '[DEBUG] FMI output variables:' )
            for type, vars in fmi_output_vars.items():
                for var in vars:
                    modules.log( '\t{0} ({1})'.format( var, type ) )

        fmi_parameters[ 'Real' ] = backend._realParameterNames
        fmi_parameters[ 'Integer' ] = backend._integerParameterNames
        fmi_parameters[ 'Boolean' ] = backend._booleanParameterNames
        fmi_parameters[ 'String' ] = backend._stringParameterNames

        if ( True == verbose ):
            modules.log( '[DEBUG] FMI parameters:' )
            for type, vars in fmi_parameters.items():
                for var in vars:
                    modules.log( '\t{0} ({1})'.format( var, type ) )

    return ( fmi_input_vars, fmi_output_vars, fmi_parameters, fmu_backend_class, fmu_backend_file )


def generateFMU(
    fmi_version,
    fmi_model_identifier,
    fmu_backend_class,
    fmu_backend_file,
    fmi_input_vars,
    fmi_output_vars,
    fmi_parameters,
    start_values,
    optional_files,
    verbose,
    litter,
    modules ):
    """
    Generate an FMU from Python code.

    :param fmi_version: FMI version
    :param fmi_model_identifier: FMI model identfier for FMU
    :param fmu_backend_class: name of the backend class
    :param fmu_backend_file: file defining the backend class
    :param fmi_input_vars: definition of input variables
    :param fmi_output_vars: definition of output variable names
    :param start_values: definition of start values
    :param optional_files: definition of additional files
    :param verbose: verbosity flag
    :param litter: do not clean-up intermediate files
    :param modules: named tuple containing all imported modules
    """
    # Create new GUID.
    guid = str( modules.uuid.uuid1() )

    # Create FMU model description.
    model_description_name = createModelDescription( fmi_version, fmi_model_identifier, fmu_backend_class, fmu_backend_file, fmi_input_vars, fmi_output_vars, fmi_parameters, guid, start_values, optional_files, verbose, modules )

    # Create script for running the backend implementation.
    backend_script_name = createBackendScript( fmu_backend_class, fmu_backend_file, guid, verbose, modules )

    # Create FMU shared library.
    fmu_shared_library_name = createSharedLibrary( fmi_model_identifier, fmi_version, verbose, modules )

    # Check if working directory for FMU creation already exists.
    if ( True == modules.os.path.isdir( fmi_model_identifier ) ):
        modules.shutil.rmtree( fmi_model_identifier, False )

    # Retrieve platform information.
    platform_type = str()
    platform_id = modules.platform.platform().lower()
    platform_bits = modules.platform.architecture()[0][:2]
    if 'linux' in platform_id:
        platform_type = 'linux' + platform_bits
    elif 'cygwin' in platform_id:
        platform_type = 'cygwin' + platform_bits
    elif 'windows' in platform_id:
        platform_type = 'win' + platform_bits
    else:
        raise RuntimeError( '\n[ERROR] platform not supported: {}'.format( modules.platform.platform() ) )

    # Working directory path for the FMU DLL.
    binaries_dir = modules.os.path.join( fmi_model_identifier, 'binaries', platform_type )

    # Create working directory (incl. sub-directories) for FMU creation.
    modules.os.makedirs( binaries_dir )

    # Resources directory path.
    resources_dir = modules.os.path.join( fmi_model_identifier, 'resources' )

    # Create resources directory for FMU creation.
    modules.os.makedirs( resources_dir )

    # Copy all files to working directory.
    modules.shutil.copy( model_description_name, fmi_model_identifier ) # XML model description.
    modules.shutil.copy( backend_script_name, resources_dir ) # Script for running the backend implementation.
    modules.shutil.copy( fmu_backend_file, resources_dir ) # backend implementation.
    for file_name in optional_files: # Additional files.
        modules.shutil.copy( file_name, resources_dir )
    modules.shutil.copy( fmu_shared_library_name, binaries_dir ) # FMU DLL.

    # Create ZIP archive.
    if ( True == modules.os.path.isfile( fmi_model_identifier + '.zip' ) ):
        modules.os.remove( fmi_model_identifier + '.zip' )
    modules.shutil.make_archive( fmi_model_identifier, 'zip', fmi_model_identifier )

    # Finally, create the FMU!!!
    fmu_file_name = fmi_model_identifier + '.fmu'
    if ( True == modules.os.path.isfile( fmu_file_name ) ):
        modules.os.remove( fmu_file_name )
    modules.os.rename( fmi_model_identifier + '.zip', fmu_file_name )

    # Clean up.
    if ( False == litter ):
        for fn in [ model_description_name, backend_script_name, 'build.log', 'fmiFunctions.obj' ]:
            modules.os.remove( fn ) if modules.os.path.isfile( fn ) else None
        modules.shutil.rmtree( fmi_model_identifier, False )
        for file_name in modules.glob.glob( fmi_model_identifier + '.*' ):
            if not ( '.fmu' in file_name or '.py' in file_name ): modules.os.remove( file_name )

    # Return name of created FMU.
    return fmu_file_name


# Create model description.
def createModelDescription(
        fmi_version,
        fmi_model_identifier,
        fmu_backend_class,
        fmu_backend_file,
        fmi_input_vars,
        fmi_output_vars,
        fmi_parameters,
        guid,
        start_values,
        optional_files,
        verbose,
        modules ):
    # Retrieve templates for different parts of XML model description according to FMI version.
    ( model_description_header, scalar_variable_node, model_description_footer ) = getModelDescriptionTemplates( fmi_version, verbose, modules )

    # FMI model identifier.
    model_description_header = model_description_header.replace( '__MODEL_IDENTIFIER__', fmi_model_identifier )

    # Model name.
    fmi_model_name = modules.os.path.basename( fmu_backend_file ).split( '.' )[0] # Deck file name with extension.
    model_description_header = model_description_header.replace( '__MODEL_NAME__', fmi_model_name )

    # Creation date and time.
    model_description_header = model_description_header.replace( '__DATE_AND_TIME__', modules.time.strftime( "%Y-%m-%dT%H:%M:%S" ) )

    # Author name.
    model_description_header = model_description_header.replace( '__USER__', modules.getpass.getuser() )

    # GUID.
    model_description_header = model_description_header.replace( '__GUID__', guid )

    # URI of Python main executable.
    python_exe_uri = modules.urlparse.urljoin( 'file:', modules.urllib.pathname2url( modules.sys.executable ) )
    model_description_header = model_description_header.replace( '__PYTHON_URI__', python_exe_uri )

    # Define a string to collect all scalar variable definitions.
    model_description_scalars = ''

    # Add scalar input variables description. Value references for inputs start with 1.
    input_val_ref = 1
    for type, vars in fmi_input_vars.items():
        for var in vars:
            scalar_variable_description = scalar_variable_node
            scalar_variable_description = addVariabilityAndCausalityToModelDescription( scalar_variable_description, fmi_version, type, True, False, verbose, modules )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_TYPE__', type )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_NAME__', var )
            scalar_variable_description = scalar_variable_description.replace( '__VAL_REF__', str( input_val_ref ) )
            scalar_variable_description = scalar_variable_description.replace( '__INITIAL__', '' )
            if var in start_values:
                if isinstance( start_values[var], bool ): start_values[var] = int( start_values[var] )
                start_value_description = ' start=\"' + str( start_values[var] ) + '\"'
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', start_value_description )
                if ( True == verbose ): modules.log( '[DEBUG] Added start value to model description: ', var, '=', start_values[var] )
            else:
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', '' )
            input_val_ref += 1
            # Write scalar variable description to file.
            model_description_scalars += scalar_variable_description;

    # Add scalar input variables description. Value references for outputs start with 1001 (except there are already input value references with corresponding values).
    output_val_ref = 1001 if ( input_val_ref < 1001 ) else input_val_ref
    for type, vars in fmi_output_vars.items():
        for var in vars:
            scalar_variable_description = scalar_variable_node
            scalar_variable_description = addVariabilityAndCausalityToModelDescription( scalar_variable_description, fmi_version, type, False, False, verbose, modules )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_TYPE__', type )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_NAME__', var )
            scalar_variable_description = scalar_variable_description.replace( '__VAL_REF__', str( output_val_ref ) )
            if var in start_values:
                if isinstance( start_values[var], bool ): start_values[var] = int( start_values[var] )
                start_value_description = ' start=\"' + str( start_values[var] ) + '\"'
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', start_value_description )
                scalar_variable_description = scalar_variable_description.replace( '__INITIAL__', 'initial="exact"' )
                if ( True == verbose ): modules.log( '[DEBUG] Added start value to model description: ', var, '=', start_values[var] )
            else:
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', '' )
                scalar_variable_description = scalar_variable_description.replace( '__INITIAL__', '' )
            output_val_ref += 1
            # Write scalar variable description to file.
            model_description_scalars += scalar_variable_description;

    # Add parameter descriptions. Value references for parameters start with 2001 (except there are already other references with corresponding values).
    param_val_ref = 2001 if ( output_val_ref < 2001 ) else output_val_ref
    for type, vars in fmi_parameters.items():
        for var in vars:
            scalar_variable_description = scalar_variable_node
            scalar_variable_description = addVariabilityAndCausalityToModelDescription( scalar_variable_description, fmi_version, type, False, True, verbose, modules )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_TYPE__', type )
            scalar_variable_description = scalar_variable_description.replace( '__VAR_NAME__', var )
            scalar_variable_description = scalar_variable_description.replace( '__VAL_REF__', str( param_val_ref ) )
            if var in start_values:
                if isinstance( start_values[var], bool ): start_values[var] = int( start_values[var] )
                start_value_description = ' start=\"' + str( start_values[var] ) + '\"'
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', start_value_description )
                scalar_variable_description = scalar_variable_description.replace( '__INITIAL__', 'initial="exact"' )
                if ( True == verbose ): modules.log( '[DEBUG] Added start value to model description: ', var, '=', start_values[var] )
            else:
                scalar_variable_description = scalar_variable_description.replace( '__START_VALUE__', '' )
                scalar_variable_description = scalar_variable_description.replace( '__INITIAL__', '' )
            param_val_ref += 1
            # Write scalar variable description to file.
            model_description_scalars += scalar_variable_description;

    # Add backend class file and optional files.
    ( model_description_header, model_description_footer ) = \
        addOptionalFilesToModelDescription( model_description_header, model_description_footer, optional_files, fmi_version, verbose, modules)

    # Create new XML model description file.
    model_description_name = 'modelDescription.xml'
    model_description = open( model_description_name, 'w' )
    model_description.write( model_description_header );
    model_description.write( model_description_scalars );
    model_description.write( model_description_footer );
    model_description.close()

    return model_description_name


# Create script for running the backend implementation.
def createBackendScript( fmu_backend_class, fmu_backend_file, guid, verbose, modules ):
    # Extract file name from full path.
    file_name = modules.os.path.basename( fmu_backend_file )

    # Define name of script.
    backend_script_name = 'run_backend_{}.py'.format( guid )

    # Create script.
    backend_script = open( backend_script_name, 'w' )
    backend_script.write( 'import os.path\n')
    backend_script.write( 'from fmipp.export.runFMUBackend import runFMUBackend\n' )
    backend_script.write( 'backend_implementation = os.path.join( os.path.dirname( __file__ ), \'{backend_file}\' )\n'.format( backend_file = file_name ) )
    backend_script.write( 'runFMUBackend( backend_implementation, \'{backend_class}\' )\n'.format( backend_class = fmu_backend_class ) )

    if ( True == verbose ): modules.log( '[DEBUG] Created script for running the backend implementation: ', backend_script_name )

    return backend_script_name


# Get templates for the XML model description depending on the FMI version.
def getModelDescriptionTemplates( fmi_version, verbose, modules ):
    if ( '1' == fmi_version ): # FMI 1.0
        return fmi1GetModelDescriptionTemplates( verbose, modules )
    elif ( '2' == fmi_version ): # FMI 2.0
        return fmi2GetModelDescriptionTemplates( verbose, modules )


# Add variability to scalar variability description.
def addVariabilityAndCausalityToModelDescription( scalar_variable_description, fmi_version, type, is_input, is_parameter, verbose, modules ):
    if ( '1' == fmi_version ): # FMI 1.0
        return fmi1addVariabilityAndCausalityToModelDescription( scalar_variable_description, type, is_input, is_parameter, verbose, modules )
    elif ( '2' == fmi_version ): # FMI 2.0
        return fmi2addVariabilityAndCausalityToModelDescription( scalar_variable_description, type, is_input, is_parameter, verbose, modules )


# Add deck file as entry point to XML model description.
def addBackendClassFileToModelDescription( fmu_backend_class, fmu_backend_file, header, footer, fmi_version, verbose, modules ):
    if ( '1' == fmi_version ): # FMI 1.0
        return fmi1AddBackendClassFileToModelDescription( fmu_backend_class, fmu_backend_file, header, footer, verbose, modules )
    elif ( '2' == fmi_version ): # FMI 2.0
        return fmi2AddBackendClassFileToModelDescription( fmu_backend_class, fmu_backend_file, header, footer, verbose, modules )


# Add optional files to XML model description.
def addOptionalFilesToModelDescription( header, footer, optional_files, fmi_version, verbose, modules ):
    if ( '1' == fmi_version ):
        return fmi1AddOptionalFilesToModelDescription( optional_files, header, footer, verbose, modules )
    if ( '2' == fmi_version ):
        return fmi2AddOptionalFilesToModelDescription( optional_files, header, footer, verbose, modules )


# Create DLL for FMU.
def createSharedLibrary( fmi_model_identifier, fmi_version, verbose, modules ):
    if ( '1' == fmi_version ):
        return fmi1CreateSharedLibrary( fmi_model_identifier, verbose, modules )
    if ( '2' == fmi_version ):
        return fmi2CreateSharedLibrary( fmi_model_identifier, verbose, modules )



# -----------------------------------------------------------------
# Copyright (c) 2022, AIT Austrian Institute of Technology GmbH.
# All rights reserved. See file FMIPP_LICENSE.txt for details.
# -----------------------------------------------------------------

try:
	import urlparse, urllib
except:
	pass
try: # python 3
	import urllib.parse as urlparse, urllib.request as urllib
except:
	pass

# Convert path to URI.
def pathToURI( path ):
	return urlparse.urljoin( 'file:', urllib.pathname2url( path ) )


# Concatenate two paths and return URI of resulting path
def concatPathsToURI( path1, path2 ):
	import os.path
	return urlparse.urljoin( 'file:', urllib.pathname2url( os.path.join( path1, path2 ) ) )



# -----------------------------------------------------------------
# Copyright (c) 2022, AIT Austrian Institute of Technology GmbH.
# All rights reserved. See file FMIPP_LICENSE.txt for details.
# -----------------------------------------------------------------

import abc
import warnings
import fmipp.export.fmippex as fmippex
from fmipp.export.FMIAdapterBase import FMIAdapterBase


class FMIAdapterV2( FMIAdapterBase ):
    """
    Abtract base class for defining FMUs for Co-Simulation compliant to FMI version 2.0.
    To implement an FMU, inherit a new class from class FMIAdapter that implements the methods init(...) and doStep(...).
    This class is a user-friendly wrapper for the functionality provided by the SWIG-generated Python bindings of the FMI++ library.
    """

    def defineRealParameters( self, *parameterNames ):
        """
        Define parameters (of type real).
        :param *parameterNames: names of parameters of type real
        """
        self._realParameterNames = list( parameterNames )
        self._realParameterSize = len( self._realParameterNames )
        self._realParameters = fmippex.new_double_array( self._realParameterSize )

        # Initialize parameters of type real.
        status = self._defineVariables(
            self._realParameterNames, self._realParameterSize,
            self._realParameters, float, self._debugRealParameterValues, 
            self._backend.initializeRealParameters if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineRealParameters] initializeRealParameters not successful' )
    

    def defineRealInputs( self, *inputVariableNames ):
        """
        Define input variables (of type real).
        :param *inputVariableNames: names of input variables of type real
        """
        self._realInputNames = list( inputVariableNames )
        self._realInputSize = len( self._realInputNames )
        self._realInputs = fmippex.new_double_array( self._realInputSize )

        # Initialize inputs (of type real).
        status = self._defineVariables(
            self._realInputNames, self._realInputSize,
            self._realInputs, float, self._debugRealInputValues, 
            self._backend.initializeRealInputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineRealInputs] initializeRealInputs not successful' )


    def defineRealOutputs( self, *outputVariableNames ):
        """
        Define output variables (of type real).
        :param *outputVariableNames: names of output variables of type real
        """
        self._realOutputNames = list( outputVariableNames )
        self._realOutputSize = len( self._realOutputNames )
        self._realOutputs = fmippex.new_double_array( self._realOutputSize )

        # Initialize outputs (of type real).
        status = self._defineVariables(
            self._realOutputNames, self._realOutputSize,
            self._realOutputs, float, self._debugRealOutputValues, 
            self._backend.initializeRealOutputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineRealOutputs] initializeRealOutputs not successful' ); end


    def defineIntegerParameters( self, *parameterNames ):
        """
        Define parameters (of type integer).        
        :param *parameterNames: names of parameters of type integer
        """
        self._integerParameterNames = list( parameterNames )
        self._integerParameterSize = len( self._integerParameterNames )
        self._integerParameters = fmippex.new_int_array( self._integerParameterSize )

        # Initialize parameters of type integer.
        status = self._defineVariables(
            self._integerParameterNames, self._integerParameterSize,
            self._integerParameters, int, self._debugIntegerParameterValues, 
            self._backend.initializeIntegerParameters if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineIntegerParameters] initializeIntegerParameters not successful' )


    def defineIntegerInputs( self, *inputVariableNames ):
        """
        Define input variables (of type integer).
        :param *inputVariableNames: names of input variables of type integer
        """
        self._integerInputNames = list( inputVariableNames )
        self._integerInputSize = len( self._integerInputNames )
        self._integerInputs = fmippex.new_int_array( self._integerInputSize )

        # Initialize inputs (of type integer).
        status = self._defineVariables(
            self._integerInputNames, self._integerInputSize,
            self._integerInputs, int, self._debugIntegerInputValues, 
            self._backend.initializeIntegerInputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineIntegerInputs] initializeIntegerInputs not successful' )


    def defineIntegerOutputs( self, *outputVariableNames ):
        """
        Define output variables (of type integer).
        :param *outputVariableNames: names of output variables of type integer
        """
        self._integerOutputNames = list( outputVariableNames )
        self._integerOutputSize = len( self._integerOutputNames )
        self._integerOutputs = fmippex.new_int_array( self._integerOutputSize )

        # Initialize outputs (of type integer).
        status = self._defineVariables(
            self._integerOutputNames, self._integerOutputSize,
            self._integerOutputs, int, self._debugIntegerOutputValues, 
            self._backend.initializeIntegerOutputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineIntegerOutputs] initializeIntegerOutputs not successful' )


    def defineBooleanParameters( self, *parameterNames ):
        """
        Define parameters (of type boolean).        
        :param *parameterNames: names of parameters of type boolean
        """
        self._booleanParameterNames = list( parameterNames )
        self._booleanParameterSize = len( self._booleanParameterNames )
        self._booleanParameters = fmippex.new_bool_array( self._booleanParameterSize )

        # Initialize parameters of type boolean.
        status = self._defineVariables(
            self._booleanParameterNames, self._booleanParameterSize,
            self._booleanParameters, bool, self._debugBooleanParameterValues, 
            self._backend.initializeBooleanParameters if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineBooleanParameters] initializeBooleanParameters not successful' )


    def defineBooleanInputs( self, *inputVariableNames ):
        """
        Define input variables (of type boolean).
        :param *inputVariableNames: names of input variables of type boolean
        """
        self._booleanInputNames = list( inputVariableNames )
        self._booleanInputSize = len( self._booleanInputNames )
        self._booleanInputs = fmippex.new_bool_array( self._booleanInputSize )

        # Initialize inputs (of type boolean).
        status = self._defineVariables(
            self._booleanInputNames, self._booleanInputSize,
            self._booleanInputs, bool, self._debugBooleanInputValues, 
            self._backend.initializeBooleanInputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineBooleanInputs] initializeBooleanInputs not successful' )


    def defineBooleanOutputs( self, *outputVariableNames ):
        """
        Define output variables (of type boolean).
        :param *outputVariableNames: names of output variables of type boolean
        """
        self._booleanOutputNames = list( outputVariableNames )
        self._booleanOutputSize = len( self._booleanOutputNames )
        self._booleanOutputs = fmippex.new_bool_array( self._booleanOutputSize )

        # Initialize outputs (of type boolean).
        status = self._defineVariables(
            self._booleanOutputNames, self._booleanOutputSize,
            self._booleanOutputs, bool, self._debugBooleanOutputValues, 
            self._backend.initializeBooleanOutputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineBooleanOutputs] initializeBooleanOutputs not successful' )


    def defineStringParameters( self, *parameterNames ):
        """
        Define Parameters (of type string).        
        :param *parameterNames: names of parameters of type string
        """
        self._stringParameterNames = list( parameterNames )
        self._stringParameterSize = len( self._stringParameterNames )
        self._stringParameters = fmippex.new_string_array( self._stringParameterSize )

        # Initialize parameters of type string.
        status = self._defineVariables(
            self._stringParameterNames, self._stringParameterSize,
            self._stringParameters, str, self._debugStringParameterValues, 
            self._backend.initializeStringParameters if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineStringParameters] initializeStringParameters not successful' )


    def defineStringInputs( self, *inputVariableNames ):
        """
        Define input variables (of type string).
        :param *inputVariableNames: names of input variables of type boolean
        """
        self._stringInputNames = list( inputVariableNames )
        self._stringInputSize = len( self._stringInputNames )
        self._stringInputs = fmippex.new_string_array( self._stringInputSize )

        # Initialize inputs (of type string).
        status = self._defineVariables(
            self._stringInputNames, self._stringInputSize,
            self._stringInputs, str, self._debugStringInputValues, 
            self._backend.initializeStringInputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( 'FMIAdapter:defineStringInputs] initializeStringInputs not successful' )


    def defineStringOutputs( self, *outputVariableNames ):
        """
        Define output variables (of type string).
        :param *outputVariableNames: names of output variables of type string
        """
        self._stringOutputNames = list( outputVariableNames )
        self._stringOutputSize = len( self._stringOutputNames )
        self._stringOutputs = fmippex.new_string_array( self._stringOutputSize )

        # Initialize outputs (of type string).
        status = self._defineVariables(
            self._stringOutputNames, self._stringOutputSize,
            self._stringOutputs, str, self._debugStringOutputValues, 
            self._backend.initializeStringOutputs if not self._backend is None else None )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:defineStringOutputs] initializeStringOutputs not successful' )


    def getRealParameterValues( self ):
        """
        Retrieve parameters (of type real).
        :return: dict of parameters of type real (includes all parameters of type real previously defined during the initialization)
        """
        # Read real parameters.
        values = self._getValues(
            self._realParameterNames, self._realParameterSize,
            self._realParameters, float, self._debugRealParameterValues,
            self._backend.getRealParameters if not self._backend is None else None,
            fmippex.double_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getRealParameterValues] getRealParameters not successful' )

        return values
        
        
    def getIntegerParameterValues( self ):
        """
        Retrieve parameters (of type integer).
        :return: dict of parameters of type integer (includes all parameters of type integer previously defined during the initialization)
        """
        # Read integer parameters.
        values = self._getValues(
            self._integerParameterNames, self._integerParameterSize,
            self._integerParameters, int, self._debugIntegerParameterValues,
            self._backend.getIntegerParameters if not self._backend is None else None,
            fmippex.int_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getIntegerParameterValues] getIntegerParameterValues not successful' )

        return values


    def getBooleanParameterValues( self ):
        """
        Retrieve parameters (of type boolean).
        :return: dict of parameters of type boolean (includes all parameters of type boolean previously defined during the initialization)
        """
        # Read boolean parameters.
        values = self._getValues(
            self._booleanParameterNames, self._booleanParameterSize,
            self._booleanParameters, bool, self._debugBooleanParameterValues,
            self._backend.getBooleanParameters if not self._backend is None else None,
            fmippex.bool_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getBooleanParameterValues] getBooleanParameterValues not successful' )

        return values


    def getStringParameterValues( self ):
        """
        Retrieve parameters (of type string).
        :return: dict of parameters of type string (includes all parameters of type string previously defined during the initialization)
        """
        # Read string parameters.
        values = self._getValues(
            self._stringParameterNames, self._stringParameterSize,
            self._stringParameters, str, self._debugStringParameterValues,
            self._backend.getStringParameters if not self._backend is None else None,
            fmippex.string_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getStringParameterValues] getStringParameterValues not successful' )

        return values


    def getRealInputValues( self ):
        """
        Retrieve input variables (of type real).
        :return: dict of input variables of type real (includes all input variables of type real previously defined during the initialization)
        """
        # Read real inputs.
        values = self._getValues(
            self._realInputNames, self._realInputSize,
            self._realInputs, float, self._debugRealInputValues,
            self._backend.getRealInputs if not self._backend is None else None,
            fmippex.double_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getRealInputValues] getRealInputValues not successful' )

        return values


    def getIntegerInputValues( self ):
        """
        Retrieve input variables (of type integer).
        :return: dict of input variables of type integer (includes all input variables of type integer previously defined during the initialization)
        """
        # Read integer inputs.
        values = self._getValues(
            self._integerInputNames, self._integerInputSize,
            self._integerInputs, int, self._debugIntegerInputValues,
            self._backend.getIntegerInputs if not self._backend is None else None,
            fmippex.int_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getIntegerInputValues] getIntegerInputValues not successful' )

        return values


    def getBooleanInputValues( self ):
        """
        Retrieve input variables (of type boolean).
        :return: dict of input variables of type boolean (includes all input variables of type boolean previously defined during the initialization)
        """
        # Read boolean inputs.
        values = self._getValues(
            self._booleanInputNames, self._booleanInputSize,
            self._booleanInputs, bool, self._debugBooleanInputValues,
            self._backend.getBooleanInputs if not self._backend is None else None,
            fmippex.bool_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getBooleanInputValues] getBooleanInputValues not successful' )

        return values


    def getStringInputValues( self ):
        """
        Retrieve input variables (of type string).
        :return: dict of input variables of type string (includes all input variables of type string previously defined during the initialization)
        """
        # Read string inputs.
        values = self._getValues(
            self._stringInputNames, self._stringInputSize,
            self._stringInputs, str, self._debugStringInputValues,
            self._backend.getStringInputs if not self._backend is None else None,
            fmippex.string_array_getitem )
        
        if values is None:
            raise RuntimeError( '[FMIAdapter:getStringInputValues] getStringInputValues not successful' )

        return values


    def setRealOutputValues( self, outputValues ):
        """
        Set output variables (of type real).
        :param outputValues: dict of output variables of type real (must include all output variables of type real previously defined during the initialization)
        """
        # Sanity check: all defined output variables have to be given as input.
        for name in self._realOutputNames:
            if not name in outputValues:
                raise RuntimeError( '[FMIAdapter:setRealOutputValues] output variable \'{}\' is missing.'.format( name ) )

        # Write current outputs.
        status = self._setValues( self._realOutputNames, self._realOutputSize, 
            self._realOutputs, outputValues, self._debugRealOutputValues,
            self._backend.setRealOutputs if not self._backend is None else None,
            fmippex.double_array_setitem )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:setRealOutputValues] setRealOutputs not successful' )
    
    
    def setIntegerOutputValues( self, outputValues ):
        """
        Set output variables (of type integer).
        :param outputValues: dict of output variables of type integer (must include all output variables of type integer previously defined during the initialization)
        """
        # Sanity check: all defined output variables have to be given as input.
        for name in self._integerOutputNames:
            if not name in outputValues:
                raise RuntimeError( '[FMIAdapter:setIntegerOutputValues] output variable \'{}\' is missing.'.format( name ) )

        # Write current outputs.
        status = self._setValues( self._integerOutputNames, self._integerOutputSize,
            self._integerOutputs, outputValues, self._debugIntegerOutputValues,
            self._backend.setIntegerOutputs if not self._backend is None else None,
            fmippex.int_array_setitem )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:setIntegerOutputValues] setIntegerOutputs not successful' )


    def setBooleanOutputValues( self, outputValues ):
        """
        Set output variables (of type boolean).
        :param outputValues: dict of output variables of type boolean (must include all output variables of type boolean previously defined during the initialization)
        """
        # Sanity check: all defined output variables have to be given as input.
        for name in self._booleanOutputNames:
            if not name in outputValues:
                raise RuntimeError( '[FMIAdapter:setBooleanOutputValues] output variable \'{}\' is missing.'.format( name ) )

        # Write current outputs.
        status = self._setValues( self._booleanOutputNames, self._booleanOutputSize,
            self._booleanOutputs, outputValues, self._debugBooleanOutputValues,
            self._backend.setBooleanOutputs if not self._backend is None else None,
            fmippex.bool_array_setitem )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:setBooleanOutputValues] setBooleanOutputs not successful' )


    def setStringOutputValues( self, outputValues ):
        """
        Set output variables (of type string). 
        :param outputValues: dict of output variables of type string (must include all output variables of type string previously defined during the initialization)
        """
        # Sanity check: all defined output variables have to be given as input.
        for name in self._stringOutputNames:
            if not name in outputValues:
                raise RuntimeError( '[FMIAdapter:setStringOutputValues] output variable \'{}\' is missing.'.format( name ) )

        # Write current outputs.
        status = self._setValues( self._stringOutputNames, self._stringOutputSize,
            self._stringOutputs, outputValues, self._debugStringOutputValues,
            self._backend.setStringOutputs if not self._backend is None else None,
            fmippex.string_array_setitem )

        if not status is fmippex.fmi2OK:
            raise RuntimeError( '[FMIAdapter:setStringOutputValues] setStringOutputs not successful' )



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _fmippex

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def new_double_pointer():
    return _fmippex.new_double_pointer()

def copy_double_pointer(value):
    return _fmippex.copy_double_pointer(value)

def delete_double_pointer(obj):
    return _fmippex.delete_double_pointer(obj)

def double_pointer_assign(obj, value):
    return _fmippex.double_pointer_assign(obj, value)

def double_pointer_value(obj):
    return _fmippex.double_pointer_value(obj)

def new_int_pointer():
    return _fmippex.new_int_pointer()

def copy_int_pointer(value):
    return _fmippex.copy_int_pointer(value)

def delete_int_pointer(obj):
    return _fmippex.delete_int_pointer(obj)

def int_pointer_assign(obj, value):
    return _fmippex.int_pointer_assign(obj, value)

def int_pointer_value(obj):
    return _fmippex.int_pointer_value(obj)

def new_string_pointer():
    return _fmippex.new_string_pointer()

def copy_string_pointer(value):
    return _fmippex.copy_string_pointer(value)

def delete_string_pointer(obj):
    return _fmippex.delete_string_pointer(obj)

def string_pointer_assign(obj, value):
    return _fmippex.string_pointer_assign(obj, value)

def string_pointer_value(obj):
    return _fmippex.string_pointer_value(obj)

def new_bool_pointer():
    return _fmippex.new_bool_pointer()

def copy_bool_pointer(value):
    return _fmippex.copy_bool_pointer(value)

def delete_bool_pointer(obj):
    return _fmippex.delete_bool_pointer(obj)

def bool_pointer_assign(obj, value):
    return _fmippex.bool_pointer_assign(obj, value)

def bool_pointer_value(obj):
    return _fmippex.bool_pointer_value(obj)

def new_double_array(nelements):
    return _fmippex.new_double_array(nelements)

def delete_double_array(ary):
    return _fmippex.delete_double_array(ary)

def double_array_getitem(ary, index):
    return _fmippex.double_array_getitem(ary, index)

def double_array_setitem(ary, index, value):
    return _fmippex.double_array_setitem(ary, index, value)

def new_int_array(nelements):
    return _fmippex.new_int_array(nelements)

def delete_int_array(ary):
    return _fmippex.delete_int_array(ary)

def int_array_getitem(ary, index):
    return _fmippex.int_array_getitem(ary, index)

def int_array_setitem(ary, index, value):
    return _fmippex.int_array_setitem(ary, index, value)

def new_string_array(nelements):
    return _fmippex.new_string_array(nelements)

def delete_string_array(ary):
    return _fmippex.delete_string_array(ary)

def string_array_getitem(ary, index):
    return _fmippex.string_array_getitem(ary, index)

def string_array_setitem(ary, index, value):
    return _fmippex.string_array_setitem(ary, index, value)

def new_bool_array(nelements):
    return _fmippex.new_bool_array(nelements)

def delete_bool_array(ary):
    return _fmippex.delete_bool_array(ary)

def bool_array_getitem(ary, index):
    return _fmippex.bool_array_getitem(ary, index)

def bool_array_setitem(ary, index, value):
    return _fmippex.bool_array_setitem(ary, index, value)
fmiModelTypesPlatform = _fmippex.fmiModelTypesPlatform

fmiPlatform = _fmippex.fmiPlatform

fmiTrue = _fmippex.fmiTrue

fmiFalse = _fmippex.fmiFalse

class fmiEventInfo(object):
    r"""Proxy of C++ fmiEventInfo class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    iterationConverged = property(_fmippex.fmiEventInfo_iterationConverged_get, _fmippex.fmiEventInfo_iterationConverged_set, doc=r"""iterationConverged""")
    stateValueReferencesChanged = property(_fmippex.fmiEventInfo_stateValueReferencesChanged_get, _fmippex.fmiEventInfo_stateValueReferencesChanged_set, doc=r"""stateValueReferencesChanged""")
    stateValuesChanged = property(_fmippex.fmiEventInfo_stateValuesChanged_get, _fmippex.fmiEventInfo_stateValuesChanged_set, doc=r"""stateValuesChanged""")
    terminateSimulation = property(_fmippex.fmiEventInfo_terminateSimulation_get, _fmippex.fmiEventInfo_terminateSimulation_set, doc=r"""terminateSimulation""")
    upcomingTimeEvent = property(_fmippex.fmiEventInfo_upcomingTimeEvent_get, _fmippex.fmiEventInfo_upcomingTimeEvent_set, doc=r"""upcomingTimeEvent""")
    nextEventTime = property(_fmippex.fmiEventInfo_nextEventTime_get, _fmippex.fmiEventInfo_nextEventTime_set, doc=r"""nextEventTime""")

    def __init__(self):
        r"""__init__(self) -> fmiEventInfo"""
        _fmippex.fmiEventInfo_swiginit(self, _fmippex.new_fmiEventInfo())
    __swig_destroy__ = _fmippex.delete_fmiEventInfo

# Register fmiEventInfo in _fmippex:
_fmippex.fmiEventInfo_swigregister(fmiEventInfo)

fmiOK = _fmippex.fmiOK

fmiWarning = _fmippex.fmiWarning

fmiDiscard = _fmippex.fmiDiscard

fmiError = _fmippex.fmiError

fmiFatal = _fmippex.fmiFatal

fmiPending = _fmippex.fmiPending

fmiDoStepStatus = _fmippex.fmiDoStepStatus

fmiPendingStatus = _fmippex.fmiPendingStatus

fmiLastSuccessfulTime = _fmippex.fmiLastSuccessfulTime

fmi2TypesPlatform = _fmippex.fmi2TypesPlatform

fmi2True = _fmippex.fmi2True

fmi2False = _fmippex.fmi2False

class fmi2EventInfo(object):
    r"""Proxy of C++ fmi2EventInfo class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    newDiscreteStatesNeeded = property(_fmippex.fmi2EventInfo_newDiscreteStatesNeeded_get, _fmippex.fmi2EventInfo_newDiscreteStatesNeeded_set, doc=r"""newDiscreteStatesNeeded""")
    terminateSimulation = property(_fmippex.fmi2EventInfo_terminateSimulation_get, _fmippex.fmi2EventInfo_terminateSimulation_set, doc=r"""terminateSimulation""")
    nominalsOfContinuousStatesChanged = property(_fmippex.fmi2EventInfo_nominalsOfContinuousStatesChanged_get, _fmippex.fmi2EventInfo_nominalsOfContinuousStatesChanged_set, doc=r"""nominalsOfContinuousStatesChanged""")
    valuesOfContinuousStatesChanged = property(_fmippex.fmi2EventInfo_valuesOfContinuousStatesChanged_get, _fmippex.fmi2EventInfo_valuesOfContinuousStatesChanged_set, doc=r"""valuesOfContinuousStatesChanged""")
    nextEventTimeDefined = property(_fmippex.fmi2EventInfo_nextEventTimeDefined_get, _fmippex.fmi2EventInfo_nextEventTimeDefined_set, doc=r"""nextEventTimeDefined""")
    nextEventTime = property(_fmippex.fmi2EventInfo_nextEventTime_get, _fmippex.fmi2EventInfo_nextEventTime_set, doc=r"""nextEventTime""")

    def __init__(self):
        r"""__init__(self) -> fmi2EventInfo"""
        _fmippex.fmi2EventInfo_swiginit(self, _fmippex.new_fmi2EventInfo())
    __swig_destroy__ = _fmippex.delete_fmi2EventInfo

# Register fmi2EventInfo in _fmippex:
_fmippex.fmi2EventInfo_swigregister(fmi2EventInfo)

fmi2OK = _fmippex.fmi2OK

fmi2Warning = _fmippex.fmi2Warning

fmi2Discard = _fmippex.fmi2Discard

fmi2Error = _fmippex.fmi2Error

fmi2Fatal = _fmippex.fmi2Fatal

fmi2Pending = _fmippex.fmi2Pending

fmi2ModelExchange = _fmippex.fmi2ModelExchange

fmi2CoSimulation = _fmippex.fmi2CoSimulation

fmi2DoStepStatus = _fmippex.fmi2DoStepStatus

fmi2PendingStatus = _fmippex.fmi2PendingStatus

fmi2LastSuccessfulTime = _fmippex.fmi2LastSuccessfulTime

fmi2Terminated = _fmippex.fmi2Terminated

fmippTypeReal = _fmippex.fmippTypeReal

fmippTypeInteger = _fmippex.fmippTypeInteger

fmippTypeBoolean = _fmippex.fmippTypeBoolean

fmippTypeString = _fmippex.fmippTypeString

fmippTypeUnknown = _fmippex.fmippTypeUnknown

fmippOK = _fmippex.fmippOK

fmippWarning = _fmippex.fmippWarning

fmippDiscard = _fmippex.fmippDiscard

fmippError = _fmippex.fmippError

fmippFatal = _fmippex.fmippFatal

fmippTrue = _fmippex.fmippTrue

fmippFalse = _fmippex.fmippFalse

fmippClockActive = _fmippex.fmippClockActive

fmippClockInactive = _fmippex.fmippClockInactive

class FMIComponentBackEnd(object):
    r"""Proxy of C++ FMIComponentBackEnd class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(self) -> FMIComponentBackEnd"""
        _fmippex.FMIComponentBackEnd_swiginit(self, _fmippex.new_FMIComponentBackEnd())
    __swig_destroy__ = _fmippex.delete_FMIComponentBackEnd

    def startInitialization(self):
        r"""startInitialization(self) -> fmippStatus"""
        return _fmippex.FMIComponentBackEnd_startInitialization(self)

    def endInitialization(self):
        r"""endInitialization(self) -> fmippStatus"""
        return _fmippex.FMIComponentBackEnd_endInitialization(self)

    def initializeRealParameters(self, *args):
        r"""
        initializeRealParameters(self, names, params) -> fmippStatus
        initializeRealParameters(self, names, params, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeRealParameters(self, *args)

    def initializeIntegerParameters(self, *args):
        r"""
        initializeIntegerParameters(self, names, params) -> fmippStatus
        initializeIntegerParameters(self, names, params, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeIntegerParameters(self, *args)

    def initializeBooleanParameters(self, *args):
        r"""
        initializeBooleanParameters(self, names, params) -> fmippStatus
        initializeBooleanParameters(self, names, params, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeBooleanParameters(self, *args)

    def initializeStringParameters(self, *args):
        r"""
        initializeStringParameters(self, names, params) -> fmippStatus
        initializeStringParameters(self, names, params, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeStringParameters(self, *args)

    def initializeRealInputs(self, *args):
        r"""
        initializeRealInputs(self, names, inputs) -> fmippStatus
        initializeRealInputs(self, names, inputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeRealInputs(self, *args)

    def initializeIntegerInputs(self, *args):
        r"""
        initializeIntegerInputs(self, names, inputs) -> fmippStatus
        initializeIntegerInputs(self, names, inputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeIntegerInputs(self, *args)

    def initializeBooleanInputs(self, *args):
        r"""
        initializeBooleanInputs(self, names, inputs) -> fmippStatus
        initializeBooleanInputs(self, names, inputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeBooleanInputs(self, *args)

    def initializeStringInputs(self, *args):
        r"""
        initializeStringInputs(self, names, inputs) -> fmippStatus
        initializeStringInputs(self, names, inputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeStringInputs(self, *args)

    def initializeRealOutputs(self, *args):
        r"""
        initializeRealOutputs(self, names, outputs) -> fmippStatus
        initializeRealOutputs(self, names, outputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeRealOutputs(self, *args)

    def initializeIntegerOutputs(self, *args):
        r"""
        initializeIntegerOutputs(self, names, outputs) -> fmippStatus
        initializeIntegerOutputs(self, names, outputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeIntegerOutputs(self, *args)

    def initializeBooleanOutputs(self, *args):
        r"""
        initializeBooleanOutputs(self, names, outputs) -> fmippStatus
        initializeBooleanOutputs(self, names, outputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeBooleanOutputs(self, *args)

    def initializeStringOutputs(self, *args):
        r"""
        initializeStringOutputs(self, names, outputs) -> fmippStatus
        initializeStringOutputs(self, names, outputs, n) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_initializeStringOutputs(self, *args)

    def waitForMaster(self):
        r"""waitForMaster(self)"""
        return _fmippex.FMIComponentBackEnd_waitForMaster(self)

    def signalToMaster(self):
        r"""signalToMaster(self)"""
        return _fmippex.FMIComponentBackEnd_signalToMaster(self)

    def getRealParameters(self, *args):
        r"""
        getRealParameters(self, parameters) -> fmippStatus
        getRealParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getRealParameters(self, *args)

    def getIntegerParameters(self, *args):
        r"""
        getIntegerParameters(self, parameters) -> fmippStatus
        getIntegerParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getIntegerParameters(self, *args)

    def getBooleanParameters(self, *args):
        r"""
        getBooleanParameters(self, parameters) -> fmippStatus
        getBooleanParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getBooleanParameters(self, *args)

    def getStringParameters(self, *args):
        r"""
        getStringParameters(self, parameters) -> fmippStatus
        getStringParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getStringParameters(self, *args)

    def setRealParameters(self, *args):
        r"""
        setRealParameters(self, parameters) -> fmippStatus
        setRealParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setRealParameters(self, *args)

    def setIntegerParameters(self, *args):
        r"""
        setIntegerParameters(self, parameters) -> fmippStatus
        setIntegerParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setIntegerParameters(self, *args)

    def setBooleanParameters(self, *args):
        r"""
        setBooleanParameters(self, parameters) -> fmippStatus
        setBooleanParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setBooleanParameters(self, *args)

    def setStringParameters(self, *args):
        r"""
        setStringParameters(self, parameters) -> fmippStatus
        setStringParameters(self, parameters, nParameters) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setStringParameters(self, *args)

    def getRealInputs(self, *args):
        r"""
        getRealInputs(self, inputs) -> fmippStatus
        getRealInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getRealInputs(self, *args)

    def getIntegerInputs(self, *args):
        r"""
        getIntegerInputs(self, inputs) -> fmippStatus
        getIntegerInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getIntegerInputs(self, *args)

    def getBooleanInputs(self, *args):
        r"""
        getBooleanInputs(self, inputs) -> fmippStatus
        getBooleanInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getBooleanInputs(self, *args)

    def getStringInputs(self, *args):
        r"""
        getStringInputs(self, inputs) -> fmippStatus
        getStringInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_getStringInputs(self, *args)

    def resetRealInputs(self, *args):
        r"""
        resetRealInputs(self, inputs) -> fmippStatus
        resetRealInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_resetRealInputs(self, *args)

    def resetIntegerInputs(self, *args):
        r"""
        resetIntegerInputs(self, inputs) -> fmippStatus
        resetIntegerInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_resetIntegerInputs(self, *args)

    def resetBooleanInputs(self, *args):
        r"""
        resetBooleanInputs(self, inputs) -> fmippStatus
        resetBooleanInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_resetBooleanInputs(self, *args)

    def resetStringInputs(self, *args):
        r"""
        resetStringInputs(self, inputs) -> fmippStatus
        resetStringInputs(self, inputs, nInputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_resetStringInputs(self, *args)

    def setRealOutputs(self, *args):
        r"""
        setRealOutputs(self, outputs) -> fmippStatus
        setRealOutputs(self, outputs, nOutputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setRealOutputs(self, *args)

    def setIntegerOutputs(self, *args):
        r"""
        setIntegerOutputs(self, outputs) -> fmippStatus
        setIntegerOutputs(self, outputs, nOutputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setIntegerOutputs(self, *args)

    def setBooleanOutputs(self, *args):
        r"""
        setBooleanOutputs(self, outputs) -> fmippStatus
        setBooleanOutputs(self, outputs, nOutputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setBooleanOutputs(self, *args)

    def setStringOutputs(self, *args):
        r"""
        setStringOutputs(self, outputs) -> fmippStatus
        setStringOutputs(self, outputs, nOutputs) -> fmippStatus
        """
        return _fmippex.FMIComponentBackEnd_setStringOutputs(self, *args)

    def enforceTimeStep(self, delta):
        r"""enforceTimeStep(self, delta)"""
        return _fmippex.FMIComponentBackEnd_enforceTimeStep(self, delta)

    def rejectStep(self):
        r"""rejectStep(self)"""
        return _fmippex.FMIComponentBackEnd_rejectStep(self)

    def logger(self, status, category, msg):
        r"""logger(self, status, category, msg)"""
        return _fmippex.FMIComponentBackEnd_logger(self, status, category, msg)

    def getCurrentCommunicationPoint(self):
        r"""getCurrentCommunicationPoint(self) -> fmippReal const &"""
        return _fmippex.FMIComponentBackEnd_getCurrentCommunicationPoint(self)

    def getCommunicationStepSize(self):
        r"""getCommunicationStepSize(self) -> fmippReal const &"""
        return _fmippex.FMIComponentBackEnd_getCommunicationStepSize(self)

    def getStopTime(self):
        r"""getStopTime(self) -> fmippReal const &"""
        return _fmippex.FMIComponentBackEnd_getStopTime(self)

    def getStopTimeDefined(self):
        r"""getStopTimeDefined(self) -> bool const &"""
        return _fmippex.FMIComponentBackEnd_getStopTimeDefined(self)

    def getLogFileName(self):
        r"""getLogFileName(self) -> fmippString"""
        return _fmippex.FMIComponentBackEnd_getLogFileName(self)

    def loggingOn(self):
        r"""loggingOn(self) -> bool"""
        return _fmippex.FMIComponentBackEnd_loggingOn(self)

    def getRealInputNames(self, names):
        r"""getRealInputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getRealInputNames(self, names)

    def getIntegerInputNames(self, names):
        r"""getIntegerInputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getIntegerInputNames(self, names)

    def getBooleanInputNames(self, names):
        r"""getBooleanInputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getBooleanInputNames(self, names)

    def getStringInputNames(self, names):
        r"""getStringInputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getStringInputNames(self, names)

    def getRealOutputNames(self, names):
        r"""getRealOutputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getRealOutputNames(self, names)

    def getIntegerOutputNames(self, names):
        r"""getIntegerOutputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getIntegerOutputNames(self, names)

    def getBooleanOutputNames(self, names):
        r"""getBooleanOutputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getBooleanOutputNames(self, names)

    def getStringOutputNames(self, names):
        r"""getStringOutputNames(self, names)"""
        return _fmippex.FMIComponentBackEnd_getStringOutputNames(self, names)

# Register FMIComponentBackEnd in _fmippex:
_fmippex.FMIComponentBackEnd_swigregister(FMIComponentBackEnd)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _fmippim

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _fmippim.delete_SwigPyIterator

    def value(self):
        return _fmippim.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _fmippim.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _fmippim.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _fmippim.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _fmippim.SwigPyIterator_equal(self, x)

    def copy(self):
        return _fmippim.SwigPyIterator_copy(self)

    def next(self):
        return _fmippim.SwigPyIterator_next(self)

    def __next__(self):
        return _fmippim.SwigPyIterator___next__(self)

    def previous(self):
        return _fmippim.SwigPyIterator_previous(self)

    def advance(self, n):
        return _fmippim.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _fmippim.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _fmippim.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _fmippim.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _fmippim.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _fmippim.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _fmippim.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _fmippim:
_fmippim.SwigPyIterator_swigregister(SwigPyIterator)


def new_double_pointer():
    return _fmippim.new_double_pointer()

def copy_double_pointer(value):
    return _fmippim.copy_double_pointer(value)

def delete_double_pointer(obj):
    return _fmippim.delete_double_pointer(obj)

def double_pointer_assign(obj, value):
    return _fmippim.double_pointer_assign(obj, value)

def double_pointer_value(obj):
    return _fmippim.double_pointer_value(obj)

def new_int_pointer():
    return _fmippim.new_int_pointer()

def copy_int_pointer(value):
    return _fmippim.copy_int_pointer(value)

def delete_int_pointer(obj):
    return _fmippim.delete_int_pointer(obj)

def int_pointer_assign(obj, value):
    return _fmippim.int_pointer_assign(obj, value)

def int_pointer_value(obj):
    return _fmippim.int_pointer_value(obj)

def new_string_pointer():
    return _fmippim.new_string_pointer()

def copy_string_pointer(value):
    return _fmippim.copy_string_pointer(value)

def delete_string_pointer(obj):
    return _fmippim.delete_string_pointer(obj)

def string_pointer_assign(obj, value):
    return _fmippim.string_pointer_assign(obj, value)

def string_pointer_value(obj):
    return _fmippim.string_pointer_value(obj)

def new_bool_pointer():
    return _fmippim.new_bool_pointer()

def copy_bool_pointer(value):
    return _fmippim.copy_bool_pointer(value)

def delete_bool_pointer(obj):
    return _fmippim.delete_bool_pointer(obj)

def bool_pointer_assign(obj, value):
    return _fmippim.bool_pointer_assign(obj, value)

def bool_pointer_value(obj):
    return _fmippim.bool_pointer_value(obj)

def new_double_array(nelements):
    return _fmippim.new_double_array(nelements)

def delete_double_array(ary):
    return _fmippim.delete_double_array(ary)

def double_array_getitem(ary, index):
    return _fmippim.double_array_getitem(ary, index)

def double_array_setitem(ary, index, value):
    return _fmippim.double_array_setitem(ary, index, value)

def new_int_array(nelements):
    return _fmippim.new_int_array(nelements)

def delete_int_array(ary):
    return _fmippim.delete_int_array(ary)

def int_array_getitem(ary, index):
    return _fmippim.int_array_getitem(ary, index)

def int_array_setitem(ary, index, value):
    return _fmippim.int_array_setitem(ary, index, value)

def new_string_array(nelements):
    return _fmippim.new_string_array(nelements)

def delete_string_array(ary):
    return _fmippim.delete_string_array(ary)

def string_array_getitem(ary, index):
    return _fmippim.string_array_getitem(ary, index)

def string_array_setitem(ary, index, value):
    return _fmippim.string_array_setitem(ary, index, value)

def new_bool_array(nelements):
    return _fmippim.new_bool_array(nelements)

def delete_bool_array(ary):
    return _fmippim.delete_bool_array(ary)

def bool_array_getitem(ary, index):
    return _fmippim.bool_array_getitem(ary, index)

def bool_array_setitem(ary, index, value):
    return _fmippim.bool_array_setitem(ary, index, value)
class StringVector(object):
    r"""Proxy of C++ std::vector< std::string > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        r"""iterator(self) -> SwigPyIterator"""
        return _fmippim.StringVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        r"""__nonzero__(self) -> bool"""
        return _fmippim.StringVector___nonzero__(self)

    def __bool__(self):
        r"""__bool__(self) -> bool"""
        return _fmippim.StringVector___bool__(self)

    def __len__(self):
        r"""__len__(self) -> std::vector< std::string >::size_type"""
        return _fmippim.StringVector___len__(self)

    def __getslice__(self, i, j):
        r"""__getslice__(self, i, j) -> StringVector"""
        return _fmippim.StringVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        r"""
        __setslice__(self, i, j)
        __setslice__(self, i, j, v)
        """
        return _fmippim.StringVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        r"""__delslice__(self, i, j)"""
        return _fmippim.StringVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        r"""
        __delitem__(self, i)
        __delitem__(self, slice)
        """
        return _fmippim.StringVector___delitem__(self, *args)

    def __getitem__(self, *args):
        r"""
        __getitem__(self, slice) -> StringVector
        __getitem__(self, i) -> std::vector< std::string >::value_type const &
        """
        return _fmippim.StringVector___getitem__(self, *args)

    def __setitem__(self, *args):
        r"""
        __setitem__(self, slice, v)
        __setitem__(self, slice)
        __setitem__(self, i, x)
        """
        return _fmippim.StringVector___setitem__(self, *args)

    def pop(self):
        r"""pop(self) -> std::vector< std::string >::value_type"""
        return _fmippim.StringVector_pop(self)

    def append(self, x):
        r"""append(self, x)"""
        return _fmippim.StringVector_append(self, x)

    def empty(self):
        r"""empty(self) -> bool"""
        return _fmippim.StringVector_empty(self)

    def size(self):
        r"""size(self) -> std::vector< std::string >::size_type"""
        return _fmippim.StringVector_size(self)

    def swap(self, v):
        r"""swap(self, v)"""
        return _fmippim.StringVector_swap(self, v)

    def begin(self):
        r"""begin(self) -> std::vector< std::string >::iterator"""
        return _fmippim.StringVector_begin(self)

    def end(self):
        r"""end(self) -> std::vector< std::string >::iterator"""
        return _fmippim.StringVector_end(self)

    def rbegin(self):
        r"""rbegin(self) -> std::vector< std::string >::reverse_iterator"""
        return _fmippim.StringVector_rbegin(self)

    def rend(self):
        r"""rend(self) -> std::vector< std::string >::reverse_iterator"""
        return _fmippim.StringVector_rend(self)

    def clear(self):
        r"""clear(self)"""
        return _fmippim.StringVector_clear(self)

    def get_allocator(self):
        r"""get_allocator(self) -> std::vector< std::string >::allocator_type"""
        return _fmippim.StringVector_get_allocator(self)

    def pop_back(self):
        r"""pop_back(self)"""
        return _fmippim.StringVector_pop_back(self)

    def erase(self, *args):
        r"""
        erase(self, pos) -> std::vector< std::string >::iterator
        erase(self, first, last) -> std::vector< std::string >::iterator
        """
        return _fmippim.StringVector_erase(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> StringVector
        __init__(self, other) -> StringVector
        __init__(self, size) -> StringVector
        __init__(self, size, value) -> StringVector
        """
        _fmippim.StringVector_swiginit(self, _fmippim.new_StringVector(*args))

    def push_back(self, x):
        r"""push_back(self, x)"""
        return _fmippim.StringVector_push_back(self, x)

    def front(self):
        r"""front(self) -> std::vector< std::string >::value_type const &"""
        return _fmippim.StringVector_front(self)

    def back(self):
        r"""back(self) -> std::vector< std::string >::value_type const &"""
        return _fmippim.StringVector_back(self)

    def assign(self, n, x):
        r"""assign(self, n, x)"""
        return _fmippim.StringVector_assign(self, n, x)

    def resize(self, *args):
        r"""
        resize(self, new_size)
        resize(self, new_size, x)
        """
        return _fmippim.StringVector_resize(self, *args)

    def insert(self, *args):
        r"""
        insert(self, pos, x) -> std::vector< std::string >::iterator
        insert(self, pos, n, x)
        """
        return _fmippim.StringVector_insert(self, *args)

    def reserve(self, n):
        r"""reserve(self, n)"""
        return _fmippim.StringVector_reserve(self, n)

    def capacity(self):
        r"""capacity(self) -> std::vector< std::string >::size_type"""
        return _fmippim.StringVector_capacity(self)
    __swig_destroy__ = _fmippim.delete_StringVector

# Register StringVector in _fmippim:
_fmippim.StringVector_swigregister(StringVector)

class UnsignedIntVector(object):
    r"""Proxy of C++ std::vector< unsigned int > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        r"""iterator(self) -> SwigPyIterator"""
        return _fmippim.UnsignedIntVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        r"""__nonzero__(self) -> bool"""
        return _fmippim.UnsignedIntVector___nonzero__(self)

    def __bool__(self):
        r"""__bool__(self) -> bool"""
        return _fmippim.UnsignedIntVector___bool__(self)

    def __len__(self):
        r"""__len__(self) -> std::vector< unsigned int >::size_type"""
        return _fmippim.UnsignedIntVector___len__(self)

    def __getslice__(self, i, j):
        r"""__getslice__(self, i, j) -> UnsignedIntVector"""
        return _fmippim.UnsignedIntVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        r"""
        __setslice__(self, i, j)
        __setslice__(self, i, j, v)
        """
        return _fmippim.UnsignedIntVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        r"""__delslice__(self, i, j)"""
        return _fmippim.UnsignedIntVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        r"""
        __delitem__(self, i)
        __delitem__(self, slice)
        """
        return _fmippim.UnsignedIntVector___delitem__(self, *args)

    def __getitem__(self, *args):
        r"""
        __getitem__(self, slice) -> UnsignedIntVector
        __getitem__(self, i) -> std::vector< unsigned int >::value_type const &
        """
        return _fmippim.UnsignedIntVector___getitem__(self, *args)

    def __setitem__(self, *args):
        r"""
        __setitem__(self, slice, v)
        __setitem__(self, slice)
        __setitem__(self, i, x)
        """
        return _fmippim.UnsignedIntVector___setitem__(self, *args)

    def pop(self):
        r"""pop(self) -> std::vector< unsigned int >::value_type"""
        return _fmippim.UnsignedIntVector_pop(self)

    def append(self, x):
        r"""append(self, x)"""
        return _fmippim.UnsignedIntVector_append(self, x)

    def empty(self):
        r"""empty(self) -> bool"""
        return _fmippim.UnsignedIntVector_empty(self)

    def size(self):
        r"""size(self) -> std::vector< unsigned int >::size_type"""
        return _fmippim.UnsignedIntVector_size(self)

    def swap(self, v):
        r"""swap(self, v)"""
        return _fmippim.UnsignedIntVector_swap(self, v)

    def begin(self):
        r"""begin(self) -> std::vector< unsigned int >::iterator"""
        return _fmippim.UnsignedIntVector_begin(self)

    def end(self):
        r"""end(self) -> std::vector< unsigned int >::iterator"""
        return _fmippim.UnsignedIntVector_end(self)

    def rbegin(self):
        r"""rbegin(self) -> std::vector< unsigned int >::reverse_iterator"""
        return _fmippim.UnsignedIntVector_rbegin(self)

    def rend(self):
        r"""rend(self) -> std::vector< unsigned int >::reverse_iterator"""
        return _fmippim.UnsignedIntVector_rend(self)

    def clear(self):
        r"""clear(self)"""
        return _fmippim.UnsignedIntVector_clear(self)

    def get_allocator(self):
        r"""get_allocator(self) -> std::vector< unsigned int >::allocator_type"""
        return _fmippim.UnsignedIntVector_get_allocator(self)

    def pop_back(self):
        r"""pop_back(self)"""
        return _fmippim.UnsignedIntVector_pop_back(self)

    def erase(self, *args):
        r"""
        erase(self, pos) -> std::vector< unsigned int >::iterator
        erase(self, first, last) -> std::vector< unsigned int >::iterator
        """
        return _fmippim.UnsignedIntVector_erase(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> UnsignedIntVector
        __init__(self, other) -> UnsignedIntVector
        __init__(self, size) -> UnsignedIntVector
        __init__(self, size, value) -> UnsignedIntVector
        """
        _fmippim.UnsignedIntVector_swiginit(self, _fmippim.new_UnsignedIntVector(*args))

    def push_back(self, x):
        r"""push_back(self, x)"""
        return _fmippim.UnsignedIntVector_push_back(self, x)

    def front(self):
        r"""front(self) -> std::vector< unsigned int >::value_type const &"""
        return _fmippim.UnsignedIntVector_front(self)

    def back(self):
        r"""back(self) -> std::vector< unsigned int >::value_type const &"""
        return _fmippim.UnsignedIntVector_back(self)

    def assign(self, n, x):
        r"""assign(self, n, x)"""
        return _fmippim.UnsignedIntVector_assign(self, n, x)

    def resize(self, *args):
        r"""
        resize(self, new_size)
        resize(self, new_size, x)
        """
        return _fmippim.UnsignedIntVector_resize(self, *args)

    def insert(self, *args):
        r"""
        insert(self, pos, x) -> std::vector< unsigned int >::iterator
        insert(self, pos, n, x)
        """
        return _fmippim.UnsignedIntVector_insert(self, *args)

    def reserve(self, n):
        r"""reserve(self, n)"""
        return _fmippim.UnsignedIntVector_reserve(self, n)

    def capacity(self):
        r"""capacity(self) -> std::vector< unsigned int >::size_type"""
        return _fmippim.UnsignedIntVector_capacity(self)
    __swig_destroy__ = _fmippim.delete_UnsignedIntVector

# Register UnsignedIntVector in _fmippim:
_fmippim.UnsignedIntVector_swigregister(UnsignedIntVector)

typeReal = _fmippim.typeReal

typeInteger = _fmippim.typeInteger

typeBoolean = _fmippim.typeBoolean

typeString = _fmippim.typeString

typeUnknown = _fmippim.typeUnknown

statusOK = _fmippim.statusOK

statusWarning = _fmippim.statusWarning

statusDiscard = _fmippim.statusDiscard

statusError = _fmippim.statusError

statusFatal = _fmippim.statusFatal

clockActive = _fmippim.clockActive

clockInactive = _fmippim.clockInactive

invalid = _fmippim.invalid

fmuTypeME1 = _fmippim.fmuTypeME1

fmuTypeCS1 = _fmippim.fmuTypeCS1

fmuTypeME2 = _fmippim.fmuTypeME2

fmuTypeCS2 = _fmippim.fmuTypeCS2

fmuTypeMEAndCS2 = _fmippim.fmuTypeMEAndCS2

class FMUModelExchangeV1(object):
    r"""Proxy of C++ fmi_1_0::FMUModelExchange class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        FMUModelExchange(fmuDirUri, modelIdentifier, loggingOn=False, stopBeforeEvent=False, eventSearchPrecision=1e-4, type=IntegratorType::bdf) -> FMUModelExchangeV1
        FMUModelExchange(modelIdentifier, loggingOn=False, stopBeforeEvent=False, eventSearchPrecision=1e-4, type=IntegratorType::bdf) -> FMUModelExchangeV1
        __init__(self, aFMU) -> FMUModelExchangeV1
        """
        _fmippim.FMUModelExchangeV1_swiginit(self, _fmippim.new_FMUModelExchangeV1(*args))
    __swig_destroy__ = _fmippim.delete_FMUModelExchangeV1

    def instantiate(self, instanceName):
        r"""instantiate(self, instanceName) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_instantiate(self, instanceName)

    def initialize(self, toleranceDefined=False, tolerance=1e-5):
        r"""initialize(self, toleranceDefined=False, tolerance=1e-5) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_initialize(self, toleranceDefined, tolerance)

    def getContinuousStates(self, val):
        r"""getContinuousStates(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_getContinuousStates(self, val)

    def setContinuousStates(self, val):
        r"""setContinuousStates(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setContinuousStates(self, val)

    def getDerivatives(self, val):
        r"""getDerivatives(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_getDerivatives(self, val)

    def getEventIndicators(self, eventsind):
        r"""getEventIndicators(self, eventsind) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_getEventIndicators(self, eventsind)

    def integrateN(self, tend, nsteps):
        r"""integrateN(self, tend, nsteps) -> fmippTime"""
        return _fmippim.FMUModelExchangeV1_integrateN(self, tend, nsteps)

    def integrate(self, tend, deltaT=1e-5):
        r"""integrate(self, tend, deltaT=1e-5) -> fmippTime"""
        return _fmippim.FMUModelExchangeV1_integrate(self, tend, deltaT)

    def stepOverEvent(self):
        r"""stepOverEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_stepOverEvent(self)

    def completedIntegratorStep(self):
        r"""completedIntegratorStep(self) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_completedIntegratorStep(self)

    def getEventFlag(self):
        r"""getEventFlag(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_getEventFlag(self)

    def setEventFlag(self, flag):
        r"""setEventFlag(self, flag)"""
        return _fmippim.FMUModelExchangeV1_setEventFlag(self, flag)

    def resetEventFlags(self):
        r"""resetEventFlags(self)"""
        return _fmippim.FMUModelExchangeV1_resetEventFlags(self)

    def getIntEvent(self):
        r"""getIntEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_getIntEvent(self)

    def getTimeEvent(self):
        r"""getTimeEvent(self) -> fmippTime"""
        return _fmippim.FMUModelExchangeV1_getTimeEvent(self)

    def raiseEvent(self):
        r"""raiseEvent(self)"""
        return _fmippim.FMUModelExchangeV1_raiseEvent(self)

    def checkEvents(self):
        r"""checkEvents(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_checkEvents(self)

    def checkStateEvent(self):
        r"""checkStateEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_checkStateEvent(self)

    def checkTimeEvent(self):
        r"""checkTimeEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_checkTimeEvent(self)

    def checkStepEvent(self):
        r"""checkStepEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_checkStepEvent(self)

    def handleEvents(self):
        r"""handleEvents(self)"""
        return _fmippim.FMUModelExchangeV1_handleEvents(self)

    def setTime(self, time):
        r"""setTime(self, time) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setTime(self, time)

    def rewindTime(self, deltaRewindTime):
        r"""rewindTime(self, deltaRewindTime)"""
        return _fmippim.FMUModelExchangeV1_rewindTime(self, deltaRewindTime)

    def getTime(self):
        r"""getTime(self) -> fmippTime"""
        return _fmippim.FMUModelExchangeV1_getTime(self)

    def getType(self, variableName):
        r"""getType(self, variableName) -> FMIPPVariableType"""
        return _fmippim.FMUModelExchangeV1_getType(self, variableName)

    def getValueRef(self, name):
        r"""getValueRef(self, name) -> fmippValueReference"""
        return _fmippim.FMUModelExchangeV1_getValueRef(self, name)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_getLastStatus(self)

    def getRealValue(self, name):
        r"""getRealValue(self, name) -> fmippReal"""
        return _fmippim.FMUModelExchangeV1_getRealValue(self, name)

    def getIntegerValue(self, name):
        r"""getIntegerValue(self, name) -> fmippInteger"""
        return _fmippim.FMUModelExchangeV1_getIntegerValue(self, name)

    def getBooleanValue(self, name):
        r"""getBooleanValue(self, name) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV1_getBooleanValue(self, name)

    def getStringValue(self, name):
        r"""getStringValue(self, name) -> fmippString"""
        return _fmippim.FMUModelExchangeV1_getStringValue(self, name)

    def setRealValue(self, name, val):
        r"""setRealValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setRealValue(self, name, val)

    def setIntegerValue(self, name, val):
        r"""setIntegerValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setIntegerValue(self, name, val)

    def setBooleanValue(self, name, val):
        r"""setBooleanValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setBooleanValue(self, name, val)

    def setStringValue(self, name, val):
        r"""setStringValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setStringValue(self, name, val)

    def nStates(self):
        r"""nStates(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV1_nStates(self)

    def nEventInds(self):
        r"""nEventInds(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV1_nEventInds(self)

    def nValueRefs(self):
        r"""nValueRefs(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV1_nValueRefs(self)

    def sendDebugMessage(self, msg):
        r"""sendDebugMessage(self, msg)"""
        return _fmippim.FMUModelExchangeV1_sendDebugMessage(self, msg)

    def logger(self, *args):
        r"""
        logger(self, status, category, msg)
        logger(self, status, category, msg)
        """
        return _fmippim.FMUModelExchangeV1_logger(self, *args)

    def getEventSearchPrecision(self):
        r"""getEventSearchPrecision(self) -> fmippTime"""
        return _fmippim.FMUModelExchangeV1_getEventSearchPrecision(self)

    def setCallbacks(self, logger, allocateMemory, freeMemory):
        r"""setCallbacks(self, logger, allocateMemory, freeMemory) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV1_setCallbacks(self, logger, allocateMemory, freeMemory)

# Register FMUModelExchangeV1 in _fmippim:
_fmippim.FMUModelExchangeV1_swigregister(FMUModelExchangeV1)

class FMUModelExchangeV2(object):
    r"""Proxy of C++ fmi_2_0::FMUModelExchange class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        FMUModelExchange(fmuDirUri, modelIdentifier, loggingOn=False, stopBeforeEvent=False, eventSearchPrecision=1e-4, type=IntegratorType::bdf) -> FMUModelExchangeV2
        FMUModelExchange(modelIdentifier, loggingOn=False, stopBeforeEvent=False, eventSearchPrecision=1e-4, type=IntegratorType::bdf) -> FMUModelExchangeV2
        __init__(self, aFMU) -> FMUModelExchangeV2
        """
        _fmippim.FMUModelExchangeV2_swiginit(self, _fmippim.new_FMUModelExchangeV2(*args))
    __swig_destroy__ = _fmippim.delete_FMUModelExchangeV2

    def instantiate(self, instanceName):
        r"""instantiate(self, instanceName) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_instantiate(self, instanceName)

    def initialize(self, toleranceDefined=False, tolerance=1e-5):
        r"""initialize(self, toleranceDefined=False, tolerance=1e-5) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_initialize(self, toleranceDefined, tolerance)

    def getContinuousStates(self, val):
        r"""getContinuousStates(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_getContinuousStates(self, val)

    def setContinuousStates(self, val):
        r"""setContinuousStates(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setContinuousStates(self, val)

    def getDerivatives(self, val):
        r"""getDerivatives(self, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_getDerivatives(self, val)

    def getDerivativesRefs(self):
        r"""getDerivativesRefs(self) -> UnsignedIntVector"""
        return _fmippim.FMUModelExchangeV2_getDerivativesRefs(self)

    def getDerivativesNames(self):
        r"""getDerivativesNames(self) -> StringVector"""
        return _fmippim.FMUModelExchangeV2_getDerivativesNames(self)

    def getJac(self, J):
        r"""getJac(self, J) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_getJac(self, J)

    def getEventIndicators(self, eventsind):
        r"""getEventIndicators(self, eventsind) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_getEventIndicators(self, eventsind)

    def integrateN(self, tend, nsteps):
        r"""integrateN(self, tend, nsteps) -> fmippTime"""
        return _fmippim.FMUModelExchangeV2_integrateN(self, tend, nsteps)

    def integrate(self, tend, deltaT=1e-5):
        r"""integrate(self, tend, deltaT=1e-5) -> fmippTime"""
        return _fmippim.FMUModelExchangeV2_integrate(self, tend, deltaT)

    def handleEvents(self):
        r"""handleEvents(self)"""
        return _fmippim.FMUModelExchangeV2_handleEvents(self)

    def completedIntegratorStep(self):
        r"""completedIntegratorStep(self) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_completedIntegratorStep(self)

    def getEventFlag(self):
        r"""getEventFlag(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_getEventFlag(self)

    def setEventFlag(self, flag):
        r"""setEventFlag(self, flag)"""
        return _fmippim.FMUModelExchangeV2_setEventFlag(self, flag)

    def resetEventFlags(self):
        r"""resetEventFlags(self)"""
        return _fmippim.FMUModelExchangeV2_resetEventFlags(self)

    def getIntEvent(self):
        r"""getIntEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_getIntEvent(self)

    def getTimeEvent(self):
        r"""getTimeEvent(self) -> fmippReal"""
        return _fmippim.FMUModelExchangeV2_getTimeEvent(self)

    def raiseEvent(self):
        r"""raiseEvent(self)"""
        return _fmippim.FMUModelExchangeV2_raiseEvent(self)

    def checkEvents(self):
        r"""checkEvents(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_checkEvents(self)

    def checkStateEvent(self):
        r"""checkStateEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_checkStateEvent(self)

    def checkTimeEvent(self):
        r"""checkTimeEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_checkTimeEvent(self)

    def checkStepEvent(self):
        r"""checkStepEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_checkStepEvent(self)

    def setTime(self, time):
        r"""setTime(self, time) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setTime(self, time)

    def rewindTime(self, deltaRewindTime):
        r"""rewindTime(self, deltaRewindTime)"""
        return _fmippim.FMUModelExchangeV2_rewindTime(self, deltaRewindTime)

    def getTime(self):
        r"""getTime(self) -> fmippTime"""
        return _fmippim.FMUModelExchangeV2_getTime(self)

    def getType(self, variableName):
        r"""getType(self, variableName) -> FMIPPVariableType"""
        return _fmippim.FMUModelExchangeV2_getType(self, variableName)

    def getValueRef(self, name):
        r"""getValueRef(self, name) -> fmippValueReference"""
        return _fmippim.FMUModelExchangeV2_getValueRef(self, name)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_getLastStatus(self)

    def getRealValue(self, name):
        r"""getRealValue(self, name) -> fmippReal"""
        return _fmippim.FMUModelExchangeV2_getRealValue(self, name)

    def getIntegerValue(self, name):
        r"""getIntegerValue(self, name) -> fmippInteger"""
        return _fmippim.FMUModelExchangeV2_getIntegerValue(self, name)

    def getBooleanValue(self, name):
        r"""getBooleanValue(self, name) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_getBooleanValue(self, name)

    def getStringValue(self, name):
        r"""getStringValue(self, name) -> fmippString"""
        return _fmippim.FMUModelExchangeV2_getStringValue(self, name)

    def setRealValue(self, name, val):
        r"""setRealValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setRealValue(self, name, val)

    def setIntegerValue(self, name, val):
        r"""setIntegerValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setIntegerValue(self, name, val)

    def setBooleanValue(self, name, val):
        r"""setBooleanValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setBooleanValue(self, name, val)

    def setStringValue(self, name, val):
        r"""setStringValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setStringValue(self, name, val)

    def nStates(self):
        r"""nStates(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV2_nStates(self)

    def getStatesRefs(self):
        r"""getStatesRefs(self) -> UnsignedIntVector"""
        return _fmippim.FMUModelExchangeV2_getStatesRefs(self)

    def getStatesNames(self):
        r"""getStatesNames(self) -> StringVector"""
        return _fmippim.FMUModelExchangeV2_getStatesNames(self)

    def nEventInds(self):
        r"""nEventInds(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV2_nEventInds(self)

    def nValueRefs(self):
        r"""nValueRefs(self) -> fmippSize"""
        return _fmippim.FMUModelExchangeV2_nValueRefs(self)

    def sendDebugMessage(self, msg):
        r"""sendDebugMessage(self, msg)"""
        return _fmippim.FMUModelExchangeV2_sendDebugMessage(self, msg)

    def logger(self, *args):
        r"""
        logger(self, status, category, msg)
        logger(self, status, category, msg)
        """
        return _fmippim.FMUModelExchangeV2_logger(self, *args)

    def getEventSearchPrecision(self):
        r"""getEventSearchPrecision(self) -> fmippReal"""
        return _fmippim.FMUModelExchangeV2_getEventSearchPrecision(self)

    def setCallbacks(self, logger, allocateMemory, freeMemory):
        r"""setCallbacks(self, logger, allocateMemory, freeMemory) -> fmippStatus"""
        return _fmippim.FMUModelExchangeV2_setCallbacks(self, logger, allocateMemory, freeMemory)

    def setComponentEnvironment(self, env):
        r"""setComponentEnvironment(self, env)"""
        return _fmippim.FMUModelExchangeV2_setComponentEnvironment(self, env)

    def stepOverEvent(self):
        r"""stepOverEvent(self) -> fmippBoolean"""
        return _fmippim.FMUModelExchangeV2_stepOverEvent(self)

# Register FMUModelExchangeV2 in _fmippim:
_fmippim.FMUModelExchangeV2_swigregister(FMUModelExchangeV2)

class FMUCoSimulationV1(object):
    r"""Proxy of C++ fmi_1_0::FMUCoSimulation class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        FMUCoSimulation(fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-9) -> FMUCoSimulationV1
        FMUCoSimulation(modelIdentifier, loggingOn=False, timeDiffResolution=1e-9) -> FMUCoSimulationV1
        __init__(self, fmu) -> FMUCoSimulationV1
        """
        _fmippim.FMUCoSimulationV1_swiginit(self, _fmippim.new_FMUCoSimulationV1(*args))
    __swig_destroy__ = _fmippim.delete_FMUCoSimulationV1

    def terminate(self):
        r"""terminate(self)"""
        return _fmippim.FMUCoSimulationV1_terminate(self)

    def instantiate(self, instanceName, timeout, visible, interactive):
        r"""instantiate(self, instanceName, timeout, visible, interactive) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_instantiate(self, instanceName, timeout, visible, interactive)

    def initialize(self, startTime, stopTimeDefined, stopTime):
        r"""initialize(self, startTime, stopTimeDefined, stopTime) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_initialize(self, startTime, stopTimeDefined, stopTime)

    def doStep(self, currentCommunicationPoint, communicationStepSize, newStep):
        r"""doStep(self, currentCommunicationPoint, communicationStepSize, newStep) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_doStep(self, currentCommunicationPoint, communicationStepSize, newStep)

    def getTime(self):
        r"""getTime(self) -> fmippReal"""
        return _fmippim.FMUCoSimulationV1_getTime(self)

    def setRealValue(self, name, val):
        r"""setRealValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_setRealValue(self, name, val)

    def setIntegerValue(self, name, val):
        r"""setIntegerValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_setIntegerValue(self, name, val)

    def setBooleanValue(self, name, val):
        r"""setBooleanValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_setBooleanValue(self, name, val)

    def setStringValue(self, name, val):
        r"""setStringValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_setStringValue(self, name, val)

    def getRealValue(self, name):
        r"""getRealValue(self, name) -> fmippReal"""
        return _fmippim.FMUCoSimulationV1_getRealValue(self, name)

    def getIntegerValue(self, name):
        r"""getIntegerValue(self, name) -> fmippInteger"""
        return _fmippim.FMUCoSimulationV1_getIntegerValue(self, name)

    def getBooleanValue(self, name):
        r"""getBooleanValue(self, name) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_getBooleanValue(self, name)

    def getStringValue(self, name):
        r"""getStringValue(self, name) -> fmippString"""
        return _fmippim.FMUCoSimulationV1_getStringValue(self, name)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_getLastStatus(self)

    def getValueRef(self, name):
        r"""getValueRef(self, name) -> fmippValueReference"""
        return _fmippim.FMUCoSimulationV1_getValueRef(self, name)

    def nStates(self):
        r"""nStates(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV1_nStates(self)

    def nEventInds(self):
        r"""nEventInds(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV1_nEventInds(self)

    def nValueRefs(self):
        r"""nValueRefs(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV1_nValueRefs(self)

    def getType(self, variableName):
        r"""getType(self, variableName) -> FMIPPVariableType"""
        return _fmippim.FMUCoSimulationV1_getType(self, variableName)

    def canHandleVariableCommunicationStepSize(self):
        r"""canHandleVariableCommunicationStepSize(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canHandleVariableCommunicationStepSize(self)

    def canHandleEvents(self):
        r"""canHandleEvents(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canHandleEvents(self)

    def canRejectSteps(self):
        r"""canRejectSteps(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canRejectSteps(self)

    def canInterpolateInputs(self):
        r"""canInterpolateInputs(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canInterpolateInputs(self)

    def maxOutputDerivativeOrder(self):
        r"""maxOutputDerivativeOrder(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV1_maxOutputDerivativeOrder(self)

    def canRunAsynchronuously(self):
        r"""canRunAsynchronuously(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canRunAsynchronuously(self)

    def canSignalEvents(self):
        r"""canSignalEvents(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canSignalEvents(self)

    def canBeInstantiatedOnlyOncePerProcess(self):
        r"""canBeInstantiatedOnlyOncePerProcess(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canBeInstantiatedOnlyOncePerProcess(self)

    def canNotUseMemoryManagementFunctions(self):
        r"""canNotUseMemoryManagementFunctions(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV1_canNotUseMemoryManagementFunctions(self)

    def sendDebugMessage(self, msg):
        r"""sendDebugMessage(self, msg)"""
        return _fmippim.FMUCoSimulationV1_sendDebugMessage(self, msg)

    def logger(self, *args):
        r"""
        logger(self, status, category, msg)
        logger(self, status, category, msg)
        """
        return _fmippim.FMUCoSimulationV1_logger(self, *args)

    def setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished):
        r"""setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV1_setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished)

# Register FMUCoSimulationV1 in _fmippim:
_fmippim.FMUCoSimulationV1_swigregister(FMUCoSimulationV1)

class FMUCoSimulationV2(object):
    r"""Proxy of C++ fmi_2_0::FMUCoSimulation class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        FMUCoSimulation(fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-9) -> FMUCoSimulationV2
        FMUCoSimulation(modelIdentifier, loggingOn=False, timeDiffResolution=1e-9) -> FMUCoSimulationV2
        __init__(self, fmu) -> FMUCoSimulationV2
        """
        _fmippim.FMUCoSimulationV2_swiginit(self, _fmippim.new_FMUCoSimulationV2(*args))
    __swig_destroy__ = _fmippim.delete_FMUCoSimulationV2

    def terminate(self):
        r"""terminate(self)"""
        return _fmippim.FMUCoSimulationV2_terminate(self)

    def instantiate(self, instanceName, timeout, visible, interactive):
        r"""instantiate(self, instanceName, timeout, visible, interactive) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_instantiate(self, instanceName, timeout, visible, interactive)

    def initialize(self, startTime, stopTimeDefined, stopTime):
        r"""initialize(self, startTime, stopTimeDefined, stopTime) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_initialize(self, startTime, stopTimeDefined, stopTime)

    def doStep(self, currentCommunicationPoint, communicationStepSize, newStep):
        r"""doStep(self, currentCommunicationPoint, communicationStepSize, newStep) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_doStep(self, currentCommunicationPoint, communicationStepSize, newStep)

    def getTime(self):
        r"""getTime(self) -> fmippReal"""
        return _fmippim.FMUCoSimulationV2_getTime(self)

    def setRealValue(self, name, val):
        r"""setRealValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_setRealValue(self, name, val)

    def setIntegerValue(self, name, val):
        r"""setIntegerValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_setIntegerValue(self, name, val)

    def setBooleanValue(self, name, val):
        r"""setBooleanValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_setBooleanValue(self, name, val)

    def setStringValue(self, name, val):
        r"""setStringValue(self, name, val) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_setStringValue(self, name, val)

    def getRealValue(self, name):
        r"""getRealValue(self, name) -> fmippReal"""
        return _fmippim.FMUCoSimulationV2_getRealValue(self, name)

    def getIntegerValue(self, name):
        r"""getIntegerValue(self, name) -> fmippInteger"""
        return _fmippim.FMUCoSimulationV2_getIntegerValue(self, name)

    def getBooleanValue(self, name):
        r"""getBooleanValue(self, name) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_getBooleanValue(self, name)

    def getStringValue(self, name):
        r"""getStringValue(self, name) -> fmippString"""
        return _fmippim.FMUCoSimulationV2_getStringValue(self, name)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_getLastStatus(self)

    def getValueRef(self, name):
        r"""getValueRef(self, name) -> fmippValueReference"""
        return _fmippim.FMUCoSimulationV2_getValueRef(self, name)

    def nStates(self):
        r"""nStates(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV2_nStates(self)

    def nEventInds(self):
        r"""nEventInds(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV2_nEventInds(self)

    def nValueRefs(self):
        r"""nValueRefs(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV2_nValueRefs(self)

    def getType(self, variableName):
        r"""getType(self, variableName) -> FMIPPVariableType"""
        return _fmippim.FMUCoSimulationV2_getType(self, variableName)

    def canHandleVariableCommunicationStepSize(self):
        r"""canHandleVariableCommunicationStepSize(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canHandleVariableCommunicationStepSize(self)

    def canHandleEvents(self):
        r"""canHandleEvents(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canHandleEvents(self)

    def canRejectSteps(self):
        r"""canRejectSteps(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canRejectSteps(self)

    def canInterpolateInputs(self):
        r"""canInterpolateInputs(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canInterpolateInputs(self)

    def maxOutputDerivativeOrder(self):
        r"""maxOutputDerivativeOrder(self) -> fmippSize"""
        return _fmippim.FMUCoSimulationV2_maxOutputDerivativeOrder(self)

    def canRunAsynchronuously(self):
        r"""canRunAsynchronuously(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canRunAsynchronuously(self)

    def canSignalEvents(self):
        r"""canSignalEvents(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canSignalEvents(self)

    def canBeInstantiatedOnlyOncePerProcess(self):
        r"""canBeInstantiatedOnlyOncePerProcess(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canBeInstantiatedOnlyOncePerProcess(self)

    def canNotUseMemoryManagementFunctions(self):
        r"""canNotUseMemoryManagementFunctions(self) -> fmippBoolean"""
        return _fmippim.FMUCoSimulationV2_canNotUseMemoryManagementFunctions(self)

    def sendDebugMessage(self, msg):
        r"""sendDebugMessage(self, msg)"""
        return _fmippim.FMUCoSimulationV2_sendDebugMessage(self, msg)

    def logger(self, *args):
        r"""
        logger(self, status, category, msg)
        logger(self, status, category, msg)
        """
        return _fmippim.FMUCoSimulationV2_logger(self, *args)

    def setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished):
        r"""setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished) -> fmippStatus"""
        return _fmippim.FMUCoSimulationV2_setCallbacks(self, logger, allocateMemory, freeMemory, stepFinished)

    def setComponentEnvironment(self, env):
        r"""setComponentEnvironment(self, env)"""
        return _fmippim.FMUCoSimulationV2_setComponentEnvironment(self, env)

# Register FMUCoSimulationV2 in _fmippim:
_fmippim.FMUCoSimulationV2_swigregister(FMUCoSimulationV2)

class LogBuffer(object):
    r"""Proxy of C++ LogBuffer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _fmippim.delete_LogBuffer

    @staticmethod
    def getLogBuffer():
        r"""getLogBuffer() -> LogBuffer"""
        return _fmippim.LogBuffer_getLogBuffer()

    def writeToBuffer(self, msg):
        r"""writeToBuffer(self, msg)"""
        return _fmippim.LogBuffer_writeToBuffer(self, msg)

    def readFromBuffer(self):
        r"""readFromBuffer(self) -> fmippString"""
        return _fmippim.LogBuffer_readFromBuffer(self)

    def clear(self):
        r"""clear(self)"""
        return _fmippim.LogBuffer_clear(self)

    def activate(self):
        r"""activate(self)"""
        return _fmippim.LogBuffer_activate(self)

    def deactivate(self):
        r"""deactivate(self)"""
        return _fmippim.LogBuffer_deactivate(self)

    def isActivated(self):
        r"""isActivated(self) -> bool"""
        return _fmippim.LogBuffer_isActivated(self)

# Register LogBuffer in _fmippim:
_fmippim.LogBuffer_swigregister(LogBuffer)

def LogBuffer_getLogBuffer():
    r"""LogBuffer_getLogBuffer() -> LogBuffer"""
    return _fmippim.LogBuffer_getLogBuffer()

integratorEU = _fmippim.integratorEU

integratorRK = _fmippim.integratorRK

integratorABM = _fmippim.integratorABM

integratorCK = _fmippim.integratorCK

integratorDP = _fmippim.integratorDP

integratorFE = _fmippim.integratorFE

integratorBS = _fmippim.integratorBS

integratorRO = _fmippim.integratorRO

integratorBDF = _fmippim.integratorBDF

integratorABM2 = _fmippim.integratorABM2

NSTEPPERS = _fmippim.NSTEPPERS

class IncrementalFMU(object):
    r"""Proxy of C++ IncrementalFMU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4, integratorType=IntegratorType::bdf) -> IncrementalFMU
        __init__(self, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4, integratorType=IntegratorType::bdf) -> IncrementalFMU
        """
        _fmippim.IncrementalFMU_swiginit(self, _fmippim.new_IncrementalFMU(*args))
    __swig_destroy__ = _fmippim.delete_IncrementalFMU

    def init(self, *args):
        r"""
        init(self, instanceName, realVariableNames, realValues, nRealVars, startTime, lookAheadHorizon, lookAheadStepSize, integratorStepSize, toleranceDefined=False, tolerance=1e-5) -> int
        init(self, instanceName, realVariableNames, realValues, nRealVars, integerVariableNames, integerValues, nIntegerVars, booleanVariableNames, booleanValues, nBooleanVars, stringVariableNames, stringValues, nStringVars, startTime, lookAheadHorizon, lookAheadStepSize, integratorStepSize, toleranceDefined=False, tolerance=1e-5) -> int
        """
        return _fmippim.IncrementalFMU_init(self, *args)

    def setIntegratorProperties(self, prop):
        r"""setIntegratorProperties(self, prop)"""
        return _fmippim.IncrementalFMU_setIntegratorProperties(self, prop)

    def getIntegratorProperties(self):
        r"""getIntegratorProperties(self) -> Integrator::Properties"""
        return _fmippim.IncrementalFMU_getIntegratorProperties(self)

    def getType(self, varName):
        r"""getType(self, varName) -> FMIPPVariableType"""
        return _fmippim.IncrementalFMU_getType(self, varName)

    def defineRealInputs(self, inputs, nInputs):
        r"""defineRealInputs(self, inputs, nInputs)"""
        return _fmippim.IncrementalFMU_defineRealInputs(self, inputs, nInputs)

    def defineIntegerInputs(self, inputs, nInputs):
        r"""defineIntegerInputs(self, inputs, nInputs)"""
        return _fmippim.IncrementalFMU_defineIntegerInputs(self, inputs, nInputs)

    def defineBooleanInputs(self, inputs, nInputs):
        r"""defineBooleanInputs(self, inputs, nInputs)"""
        return _fmippim.IncrementalFMU_defineBooleanInputs(self, inputs, nInputs)

    def defineStringInputs(self, inputs, nInputs):
        r"""defineStringInputs(self, inputs, nInputs)"""
        return _fmippim.IncrementalFMU_defineStringInputs(self, inputs, nInputs)

    def defineRealOutputs(self, outputs, nOutputs):
        r"""defineRealOutputs(self, outputs, nOutputs)"""
        return _fmippim.IncrementalFMU_defineRealOutputs(self, outputs, nOutputs)

    def defineIntegerOutputs(self, outputs, nOutputs):
        r"""defineIntegerOutputs(self, outputs, nOutputs)"""
        return _fmippim.IncrementalFMU_defineIntegerOutputs(self, outputs, nOutputs)

    def defineBooleanOutputs(self, outputs, nOutputs):
        r"""defineBooleanOutputs(self, outputs, nOutputs)"""
        return _fmippim.IncrementalFMU_defineBooleanOutputs(self, outputs, nOutputs)

    def defineStringOutputs(self, outputs, nOutputs):
        r"""defineStringOutputs(self, outputs, nOutputs)"""
        return _fmippim.IncrementalFMU_defineStringOutputs(self, outputs, nOutputs)

    def getRealOutputs(self):
        r"""getRealOutputs(self) -> fmippReal *"""
        return _fmippim.IncrementalFMU_getRealOutputs(self)

    def getIntegerOutputs(self):
        r"""getIntegerOutputs(self) -> fmippInteger *"""
        return _fmippim.IncrementalFMU_getIntegerOutputs(self)

    def getBooleanOutputs(self):
        r"""getBooleanOutputs(self) -> fmippBoolean *"""
        return _fmippim.IncrementalFMU_getBooleanOutputs(self)

    def getStringOutputs(self):
        r"""getStringOutputs(self) -> fmippString *"""
        return _fmippim.IncrementalFMU_getStringOutputs(self)

    def sync(self, *args):
        r"""
        sync(self, t0, t1) -> fmippTime
        sync(self, t0, t1, realInputs, integerInputs=None, booleanInputs=None, stringInputs=None) -> fmippTime
        """
        return _fmippim.IncrementalFMU_sync(self, *args)

    def updateState(self, t1):
        r"""updateState(self, t1) -> fmippTime"""
        return _fmippim.IncrementalFMU_updateState(self, t1)

    def updateStateFromTheRight(self, t1):
        r"""updateStateFromTheRight(self, t1) -> fmippTime"""
        return _fmippim.IncrementalFMU_updateStateFromTheRight(self, t1)

    def syncState(self, t1, realInputs, integerInputs, booleanInputs, stringInputs):
        r"""syncState(self, t1, realInputs, integerInputs, booleanInputs, stringInputs)"""
        return _fmippim.IncrementalFMU_syncState(self, t1, realInputs, integerInputs, booleanInputs, stringInputs)

    def predictState(self, t1):
        r"""predictState(self, t1) -> fmippTime"""
        return _fmippim.IncrementalFMU_predictState(self, t1)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.IncrementalFMU_getLastStatus(self)

    def getTimeDiffResolution(self):
        r"""getTimeDiffResolution(self) -> fmippTime"""
        return _fmippim.IncrementalFMU_getTimeDiffResolution(self)

# Register IncrementalFMU in _fmippim:
_fmippim.IncrementalFMU_swigregister(IncrementalFMU)

class RollbackFMU(object):
    r"""Proxy of C++ RollbackFMU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4, integratorType=IntegratorType::bdf) -> RollbackFMU"""
        _fmippim.RollbackFMU_swiginit(self, _fmippim.new_RollbackFMU(*args))
    __swig_destroy__ = _fmippim.delete_RollbackFMU

    def integrate(self, *args):
        r"""
        integrate(self, tstop, nsteps) -> fmippReal
        integrate(self, tstop, deltaT=1E-5) -> fmippReal
        """
        return _fmippim.RollbackFMU_integrate(self, *args)

    def saveCurrentStateForRollback(self):
        r"""saveCurrentStateForRollback(self)"""
        return _fmippim.RollbackFMU_saveCurrentStateForRollback(self)

    def releaseRollbackState(self):
        r"""releaseRollbackState(self)"""
        return _fmippim.RollbackFMU_releaseRollbackState(self)

    def getRealValue(self, name):
        r"""getRealValue(self, name) -> fmippReal"""
        return _fmippim.RollbackFMU_getRealValue(self, name)

    def getIntegerValue(self, name):
        r"""getIntegerValue(self, name) -> fmippInteger"""
        return _fmippim.RollbackFMU_getIntegerValue(self, name)

    def getBooleanValue(self, name):
        r"""getBooleanValue(self, name) -> fmippBoolean"""
        return _fmippim.RollbackFMU_getBooleanValue(self, name)

    def getStringValue(self, name):
        r"""getStringValue(self, name) -> fmippString"""
        return _fmippim.RollbackFMU_getStringValue(self, name)

    def getTime(self):
        r"""getTime(self) -> fmippTime"""
        return _fmippim.RollbackFMU_getTime(self)

    def initialize(self, toleranceDefined=False, tolerance=1e-5):
        r"""initialize(self, toleranceDefined=False, tolerance=1e-5) -> fmippStatus"""
        return _fmippim.RollbackFMU_initialize(self, toleranceDefined, tolerance)

    def instantiate(self, instanceName):
        r"""instantiate(self, instanceName) -> fmippStatus"""
        return _fmippim.RollbackFMU_instantiate(self, instanceName)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.RollbackFMU_getLastStatus(self)

# Register RollbackFMU in _fmippim:
_fmippim.RollbackFMU_swigregister(RollbackFMU)

class FixedStepSizeFMU(object):
    r"""Proxy of C++ FixedStepSizeFMU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4):
        r"""__init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4) -> FixedStepSizeFMU"""
        _fmippim.FixedStepSizeFMU_swiginit(self, _fmippim.new_FixedStepSizeFMU(fmuDirUri, modelIdentifier, loggingOn, timeDiffResolution))
    __swig_destroy__ = _fmippim.delete_FixedStepSizeFMU

    def init(self, *args):
        r"""
        init(self, instanceName, realVariableNames, realValues, nRealVars, startTime, communicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        init(self, instanceName, realVariableNames, realValues, nRealVars, integerVariableNames, integerValues, nIntegerVars, booleanVariableNames, booleanValues, nBooleanVars, stringVariableNames, stringValues, nStringVars, startTime, communicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        """
        return _fmippim.FixedStepSizeFMU_init(self, *args)

    def defineRealInputs(self, inputs, nInputs):
        r"""defineRealInputs(self, inputs, nInputs)"""
        return _fmippim.FixedStepSizeFMU_defineRealInputs(self, inputs, nInputs)

    def defineIntegerInputs(self, inputs, nInputs):
        r"""defineIntegerInputs(self, inputs, nInputs)"""
        return _fmippim.FixedStepSizeFMU_defineIntegerInputs(self, inputs, nInputs)

    def defineBooleanInputs(self, inputs, nInputs):
        r"""defineBooleanInputs(self, inputs, nInputs)"""
        return _fmippim.FixedStepSizeFMU_defineBooleanInputs(self, inputs, nInputs)

    def defineStringInputs(self, inputs, nInputs):
        r"""defineStringInputs(self, inputs, nInputs)"""
        return _fmippim.FixedStepSizeFMU_defineStringInputs(self, inputs, nInputs)

    def defineRealOutputs(self, outputs, nOutputs):
        r"""defineRealOutputs(self, outputs, nOutputs)"""
        return _fmippim.FixedStepSizeFMU_defineRealOutputs(self, outputs, nOutputs)

    def defineIntegerOutputs(self, outputs, nOutputs):
        r"""defineIntegerOutputs(self, outputs, nOutputs)"""
        return _fmippim.FixedStepSizeFMU_defineIntegerOutputs(self, outputs, nOutputs)

    def defineBooleanOutputs(self, outputs, nOutputs):
        r"""defineBooleanOutputs(self, outputs, nOutputs)"""
        return _fmippim.FixedStepSizeFMU_defineBooleanOutputs(self, outputs, nOutputs)

    def defineStringOutputs(self, outputs, nOutputs):
        r"""defineStringOutputs(self, outputs, nOutputs)"""
        return _fmippim.FixedStepSizeFMU_defineStringOutputs(self, outputs, nOutputs)

    def getRealOutputs(self):
        r"""getRealOutputs(self) -> fmippReal *"""
        return _fmippim.FixedStepSizeFMU_getRealOutputs(self)

    def getIntegerOutputs(self):
        r"""getIntegerOutputs(self) -> fmippInteger *"""
        return _fmippim.FixedStepSizeFMU_getIntegerOutputs(self)

    def getBooleanOutputs(self):
        r"""getBooleanOutputs(self) -> fmippBoolean *"""
        return _fmippim.FixedStepSizeFMU_getBooleanOutputs(self)

    def getStringOutputs(self):
        r"""getStringOutputs(self) -> fmippString *"""
        return _fmippim.FixedStepSizeFMU_getStringOutputs(self)

    def sync(self, *args):
        r"""
        sync(self, t0, t1) -> fmippTime
        sync(self, t0, t1, realInputs, integerInputs, booleanInputs, stringInputs, iterateOnce=False) -> fmippTime
        """
        return _fmippim.FixedStepSizeFMU_sync(self, *args)

    def iterateOnce(self):
        r"""iterateOnce(self)"""
        return _fmippim.FixedStepSizeFMU_iterateOnce(self)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.FixedStepSizeFMU_getLastStatus(self)

# Register FixedStepSizeFMU in _fmippim:
_fmippim.FixedStepSizeFMU_swigregister(FixedStepSizeFMU)

class VariableStepSizeFMU(object):
    r"""Proxy of C++ VariableStepSizeFMU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4):
        r"""__init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4) -> VariableStepSizeFMU"""
        _fmippim.VariableStepSizeFMU_swiginit(self, _fmippim.new_VariableStepSizeFMU(fmuDirUri, modelIdentifier, loggingOn, timeDiffResolution))
    __swig_destroy__ = _fmippim.delete_VariableStepSizeFMU

    def init(self, *args):
        r"""
        init(self, instanceName, realVariableNames, realValues, nRealVars, startTime, defaultCommunicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        init(self, instanceName, realVariableNames, realValues, nRealVars, integerVariableNames, integerValues, nIntegerVars, booleanVariableNames, booleanValues, nBooleanVars, stringVariableNames, stringValues, nStringVars, startTime, communicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        """
        return _fmippim.VariableStepSizeFMU_init(self, *args)

    def defineRealInputs(self, inputs, nInputs):
        r"""defineRealInputs(self, inputs, nInputs)"""
        return _fmippim.VariableStepSizeFMU_defineRealInputs(self, inputs, nInputs)

    def defineIntegerInputs(self, inputs, nInputs):
        r"""defineIntegerInputs(self, inputs, nInputs)"""
        return _fmippim.VariableStepSizeFMU_defineIntegerInputs(self, inputs, nInputs)

    def defineBooleanInputs(self, inputs, nInputs):
        r"""defineBooleanInputs(self, inputs, nInputs)"""
        return _fmippim.VariableStepSizeFMU_defineBooleanInputs(self, inputs, nInputs)

    def defineStringInputs(self, inputs, nInputs):
        r"""defineStringInputs(self, inputs, nInputs)"""
        return _fmippim.VariableStepSizeFMU_defineStringInputs(self, inputs, nInputs)

    def defineRealOutputs(self, outputs, nOutputs):
        r"""defineRealOutputs(self, outputs, nOutputs)"""
        return _fmippim.VariableStepSizeFMU_defineRealOutputs(self, outputs, nOutputs)

    def defineIntegerOutputs(self, outputs, nOutputs):
        r"""defineIntegerOutputs(self, outputs, nOutputs)"""
        return _fmippim.VariableStepSizeFMU_defineIntegerOutputs(self, outputs, nOutputs)

    def defineBooleanOutputs(self, outputs, nOutputs):
        r"""defineBooleanOutputs(self, outputs, nOutputs)"""
        return _fmippim.VariableStepSizeFMU_defineBooleanOutputs(self, outputs, nOutputs)

    def defineStringOutputs(self, outputs, nOutputs):
        r"""defineStringOutputs(self, outputs, nOutputs)"""
        return _fmippim.VariableStepSizeFMU_defineStringOutputs(self, outputs, nOutputs)

    def getRealOutputs(self):
        r"""getRealOutputs(self) -> fmippReal *"""
        return _fmippim.VariableStepSizeFMU_getRealOutputs(self)

    def getIntegerOutputs(self):
        r"""getIntegerOutputs(self) -> fmippInteger *"""
        return _fmippim.VariableStepSizeFMU_getIntegerOutputs(self)

    def getBooleanOutputs(self):
        r"""getBooleanOutputs(self) -> fmippBoolean *"""
        return _fmippim.VariableStepSizeFMU_getBooleanOutputs(self)

    def getStringOutputs(self):
        r"""getStringOutputs(self) -> fmippString *"""
        return _fmippim.VariableStepSizeFMU_getStringOutputs(self)

    def sync(self, *args):
        r"""
        sync(self, t0, t1) -> fmippTime
        sync(self, t0, t1, realInputs, integerInputs, booleanInputs, stringInputs, iterateOnce=False) -> fmippTime
        """
        return _fmippim.VariableStepSizeFMU_sync(self, *args)

    def iterateOnce(self):
        r"""iterateOnce(self)"""
        return _fmippim.VariableStepSizeFMU_iterateOnce(self)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.VariableStepSizeFMU_getLastStatus(self)

# Register VariableStepSizeFMU in _fmippim:
_fmippim.VariableStepSizeFMU_swigregister(VariableStepSizeFMU)

class InterpolatingFixedStepSizeFMU(object):
    r"""Proxy of C++ InterpolatingFixedStepSizeFMU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4):
        r"""__init__(self, fmuDirUri, modelIdentifier, loggingOn=False, timeDiffResolution=1e-4) -> InterpolatingFixedStepSizeFMU"""
        _fmippim.InterpolatingFixedStepSizeFMU_swiginit(self, _fmippim.new_InterpolatingFixedStepSizeFMU(fmuDirUri, modelIdentifier, loggingOn, timeDiffResolution))
    __swig_destroy__ = _fmippim.delete_InterpolatingFixedStepSizeFMU

    def init(self, *args):
        r"""
        init(self, instanceName, realVariableNames, realValues, nRealVars, startTime, communicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        init(self, instanceName, realVariableNames, realValues, nRealVars, integerVariableNames, integerValues, nIntegerVars, booleanVariableNames, booleanValues, nBooleanVars, stringVariableNames, stringValues, nStringVars, startTime, communicationStepSize, stopTimeDefined=False, stopTime=std::numeric_limits< fmippTime >::infinity(), timeout=0., visible=False, interactive=False) -> int
        """
        return _fmippim.InterpolatingFixedStepSizeFMU_init(self, *args)

    def defineRealInputs(self, inputs, nInputs):
        r"""defineRealInputs(self, inputs, nInputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineRealInputs(self, inputs, nInputs)

    def defineIntegerInputs(self, inputs, nInputs):
        r"""defineIntegerInputs(self, inputs, nInputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineIntegerInputs(self, inputs, nInputs)

    def defineBooleanInputs(self, inputs, nInputs):
        r"""defineBooleanInputs(self, inputs, nInputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineBooleanInputs(self, inputs, nInputs)

    def defineStringInputs(self, inputs, nInputs):
        r"""defineStringInputs(self, inputs, nInputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineStringInputs(self, inputs, nInputs)

    def defineRealOutputs(self, outputs, nOutputs):
        r"""defineRealOutputs(self, outputs, nOutputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineRealOutputs(self, outputs, nOutputs)

    def defineIntegerOutputs(self, outputs, nOutputs):
        r"""defineIntegerOutputs(self, outputs, nOutputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineIntegerOutputs(self, outputs, nOutputs)

    def defineBooleanOutputs(self, outputs, nOutputs):
        r"""defineBooleanOutputs(self, outputs, nOutputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineBooleanOutputs(self, outputs, nOutputs)

    def defineStringOutputs(self, outputs, nOutputs):
        r"""defineStringOutputs(self, outputs, nOutputs)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_defineStringOutputs(self, outputs, nOutputs)

    def getRealOutputs(self):
        r"""getRealOutputs(self) -> fmippReal *"""
        return _fmippim.InterpolatingFixedStepSizeFMU_getRealOutputs(self)

    def getIntegerOutputs(self):
        r"""getIntegerOutputs(self) -> fmippInteger *"""
        return _fmippim.InterpolatingFixedStepSizeFMU_getIntegerOutputs(self)

    def getBooleanOutputs(self):
        r"""getBooleanOutputs(self) -> fmippBoolean *"""
        return _fmippim.InterpolatingFixedStepSizeFMU_getBooleanOutputs(self)

    def getStringOutputs(self):
        r"""getStringOutputs(self) -> fmippString *"""
        return _fmippim.InterpolatingFixedStepSizeFMU_getStringOutputs(self)

    def sync(self, *args):
        r"""
        sync(self, t0, t1) -> fmippTime
        sync(self, t0, t1, realInputs, integerInputs, booleanInputs, stringInputs, iterateOnce=False) -> fmippTime
        """
        return _fmippim.InterpolatingFixedStepSizeFMU_sync(self, *args)

    def iterateOnce(self):
        r"""iterateOnce(self)"""
        return _fmippim.InterpolatingFixedStepSizeFMU_iterateOnce(self)

    def getLastStatus(self):
        r"""getLastStatus(self) -> fmippStatus"""
        return _fmippim.InterpolatingFixedStepSizeFMU_getLastStatus(self)

# Register InterpolatingFixedStepSizeFMU in _fmippim:
_fmippim.InterpolatingFixedStepSizeFMU_swigregister(InterpolatingFixedStepSizeFMU)






# -----------------------------------------------------------------
# Copyright (c) 2022, AIT Austrian Institute of Technology GmbH.
# All rights reserved. See file FMIPP_LICENSE.txt for details.
# -----------------------------------------------------------------

# Simplify an FMU's XML model description by removing all scalar variables whose causality is not defined as either 'input' or 'output'. Also remove type definitions that are not related to input or output types.
def simplifyModelDescription( xmlModelDescription ):
	import xml.etree.ElementTree as et

	# parse model description and retrieve root element
	root = et.fromstring( xmlModelDescription )

	# get XML node listing the model variables
	modelVariables = root.find( "ModelVariables" )

	# get list of all XML nodes representing scalar variables
	scalars = modelVariables.findall( "./ScalarVariable" )

	# set of declared types of scalar variables that are not removed
	declaredTypes = set()

	# iterate through all XML nodes representing scalar variables
	for elem in scalars:
		# check if node defines attribute 'causality'
		if 'causality' in elem.attrib:
			# get value of attribute causality
			causality = elem.attrib['causality'] 
			# remove node if causility is neither 'input' nor 'output'
			if not ( causality == 'input' or causality == 'output' ):
				modelVariables.remove( elem );
			else:
				# save type of scalar variable
				value = elem.findall( ".//*[@declaredType]" )
				if len( value ) > 1:
					print( "WARNING: scalar variable has more than one type defined" )
					declaredTypes.add( value[0].attrib['declaredType'] )
				else:
					# attribute 'causality' not defined --> delete XML node
					modelVariables.remove( elem );

	# get XML node listing the type definitions
	typeDefinitions = root.find( "TypeDefinitions" )

	# get list of all XML nodes representing types
	types = typeDefinitions.findall( "./Type" )

	# iterate through all XML nodes representing types
	for elem in types:
		# remove node if the type it declares is not used by remaining scalar variables
		if elem.attrib['name'] not in declaredTypes:
			typeDefinitions.remove( elem )

	# return simpified XML model description as string
	return et.tostring( root )


# Simplify an FMU by removing all scalar variables whose causality is not defined as either 'input' or 'output' from its XML model description. Also remove type definitions that are not related to input or output types. Attention: The changes applied to the FMU are permanent.
def simplifyFMU( fmuFileName ):
	import sys, os, zipfile, zlib

	# check if specified file is indeed a zip file
	if not zipfile.is_zipfile( fmuFileName ):
		print( '%s is not a valid ZIP archive' % fmuFileName )
		sys.exit()

	# access FMU
	fmuIn = zipfile.ZipFile( fmuFileName, 'r' )

	# output zip archive
	tmpFmuFileName = fmuFileName + '.tmp'
	fmuOut = zipfile.ZipFile( tmpFmuFileName, 'w', compression = zipfile.ZIP_DEFLATED )

	try:
		# access FMU model description
		xmlModelDescription = fmuIn.read( 'modelDescription.xml' )

		# simplify model description
		simplifiedModelDesciption = simplifyModelDescription( xmlModelDescription )

		# iterate through content of FMU
		for item in fmuIn.infolist():
			# write all content to new archive, but use the simlified model description
			if not item.filename == 'modelDescription.xml':
				buffer = fmuIn.read( item.filename )
				fmuOut.writestr( item, buffer )
			else:
				fmuOut.writestr( 'modelDescription.xml', simplifiedModelDesciption )
	except KeyError:
		print( '\nInvalid FMU: no modelDescription.xml found in %s\n' % fmuFileName )

	# print info about simplified FMU
	import datetime
	for info in fmuOut.infolist():
		print( info.filename )
		print( '\tComment:\t', info.comment )
		print( '\tModified:\t', datetime.datetime(*info.date_time) )
		print( '\tSystem:\t\t', info.create_system, '(0 = Windows, 3 = Unix)' )
		print( '\tZIP version:\t', info.create_version )
		print( '\tCompressed:\t', info.compress_size, 'bytes' )
		print( '\tUncompressed:\t', info.file_size, 'bytes' )
		print()

	# close zip archives
	fmuIn.close()
	fmuOut.close()

	# replace original FMU with simplified FMU
	os.remove( fmuFileName );
	os.rename( tmpFmuFileName, fmuFileName )



if __name__ == '__main__':

	if len( sys.argv ) != 2:
		print( 'Usage:\n\tpython simplifyModelDescription.py <fmu-file-name>\n' )
		sys.exit()

	fmuFileName = sys.argv[1]

	simplifyFMU( fmuFileName )

